#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail.translations.helpers import (
    plural2,
    ca_join,
    es_join,
    en_join
)

# FacebookPublicationExtension
#------------------------------------------------------------------------------
translations.define("FacebookPublicationExtension.targets",
    ca = u"Destins",
    es = u"Destinos",
    en = u"Destinations"
)

# FacebookPublicationTarget
#------------------------------------------------------------------------------
translations.define("FacebookPublicationTarget",
    ca = u"Destí de publicació Facebook",
    es = u"Destino de publicación Facebook",
    en = u"Facebook publication target"
)

translations.define("FacebookPublicationTarget-plural",
    ca = u"Destins de publicació Facebook",
    es = u"Destinos de publicación Facebook",
    en = u"Facebook publication targets"
)

translations.define("FacebookPublicationTarget.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("FacebookPublicationTarget.graph_object_id",
    ca = u"Objecte destí",
    es = u"Objeto destino",
    en = u"Target object"
)

translations.define("FacebookPublicationTarget.graph_object_id-explanation",
    ca = u"Identificador de l'objecte de Facebook (usuari, pàgina, etc.) on "
         u"es volen publicar els continguts",
    es = u"Identificador del objeto de Facebook (usuario, página, etc.) "
         u"donde se quieren publicar los contenidos",
    en = u"Identifier for the Facebook object (user, page, etc) where the "
         u"content is to be published"
)

translations.define("FacebookPublicationTarget.administrator_id",
    ca = u"Administrador de la pàgina",
    es = u"Administrador de la página",
    en = u"Page administrator"
)

translations.define("FacebookPublicationTarget.administrator_id-explanation",
    ca = u"Si l'objecte destí és una pàgina de Facebook, cal introduir "
         u"l'identificador de l'usuari de Facebook que la gestiona. En cas "
         u"contrari, deixar en blanc.",
    es = u"Si el objeto destino es una página de Facebook, es necesario "
         u"introducir el identificador del usuario de Facebook que la "
         u"gestiona. En caso contrario, dejar en blanco.",
    en = u"If the target object is a Facebook page, introduce the identifier "
         u"of its appointed administrator. Otherwise, leave blank."
)

translations.define("FacebookPublicationTarget.app_id",
    ca = u"Id d'aplicació Facebook",
    es = u"Id de aplicación Facebook",
    en = u"Facebook application ID"
)

translations.define("FacebookPublicationTarget.app_secret",
    ca = u"Codi secret d'aplicació Facebook",
    es = u"Código secreto de aplicación Facebook",
    en = u"Facebook application secret"
)

translations.define("FacebookPublicationTarget.auth_token",
    ca = u"Autorització",
    es = u"Autorización",
    en = u"Authorization"
)

translations.define("FacebookPublicationTarget.auth_token-conceded",
    ca = u"Concedida",
    es = u"Concedida",
    en = u"Granted"
)

translations.define("FacebookPublicationTarget.auth_token-pending",
    ca = u"Pendent",
    es = u"Pendiente",
    en = u"Pending"
)

translations.define("FacebookPublicationTarget.languages",
    ca = u"Idiomes en que es publica",
    es = u"Idiomas en los que se publica",
    en = u"Published languages"
)

translations.define("FacebookPublicationTarget.targeting",
    ca = u"Segmentació per regió/idioma",
    es = u"Segmentación por región/idioma",
    en = u"Regional/language targeting"
)

# User actions
#------------------------------------------------------------------------------
translations.define("Action fbpublish",
    ca = u"Publicar a Facebook",
    es = u"Publicar en Facebook",
    en = u"Publish to Facebook"
)

translations.define("Action fbpublish_albums",
    ca = u"Publicar imatges a Facebook",
    es = u"Publicar imágenes en Facebook",
    en = u"Publish pictures to Facebook"
)

translations.define("Action fbpublish_auth",
    ca = u"Autoritzar",
    es = u"Autorizar",
    en = u"Authorize"
)

translations.define(
    "woost.extensions.facebookauthentication.fbpublish_auth.success",
    ca = u"S'ha completat l'autorització amb Facebook de forma "
         u"satisfactòria.",
    es = u"Se ha completado la autorización con Facebook de forma "
         u"satisfactoria.",
    en = u"Facebook authorization successful."
)

translations.define(
    "woost.extensions.facebookauthentication.fbpublish_auth.error",
    ca = u"L'autorització amb Facebook ha fallat."
         u"<p><em>Error:</em> %(error)s</p>"
         u"<p><em>Raó:</em> %(error_reason)s</p>"
         u"<p><em>Descripció:</em> %(error_description)s</p>",
    es = u"La autorización con Facebook ha fallado."
         u"<p><em>Error:</em> %(error)s</p>"
         u"<p><em>Razón:</em> %(error_reason)s</p>"
         u"<p><em>Descripción:</em> %(error_description)s</p>",
    en = u"Facebook authorization failed."
         u"<p><em>Error:</em> %(error)s</p>"
         u"<p><em>Reason:</em> %(error_reason)s</p>"
         u"<p><em>Description:</em> %(error_description)s</p>"
)

translations.define(
    "woost.extensions.facebookauthentication.fbpublish_auth.oauth_error",
    ca = u"L'autorització ha fallat: revisa l'id i el codi secret de "
        u"l'aplicació. Resposta: %(response)s",
    es = u"La autorización ha fallado: revisa el id y código secreto de tu "
         u"aplicación. Respuesta: %(response)s",
    en = u"Authorization failed: check your application id and secret code. "
         u"Response: %(response)s"
)

# FacebookPublicationPermission
#------------------------------------------------------------------------------
translations.define("FacebookPublicationPermission",
    ca = u"Permís de publicació a Facebook",
    es = u"Permiso de publicación en Facebook",
    en = u"Facebook publication permission"
)

translations.define("FacebookPublicationPermission-plural",
    ca = u"Permisos de publicació a Facebook",
    es = u"Permisos de publicación en Facebook",
    en = u"Facebook publication permissions"
)

translations.define("FacebookPublicationPermission.publication_targets",
    ca = u"Destins de publicació",
    es = u"Destinos de publicación",
    en = u"Publication targets"
)

# FacebookPublicationController
#------------------------------------------------------------------------------

translations.define("FacebookPublicationForm.subset",
    ca = u"Contingut a publicar",
    es = u"Contenido a publicar",
    en = u"Published content"
)

translations.define("FacebookPublicationForm.published_languages",
    ca = u"Idiomes en que es publicarà el contingut",
    es = u"Idiomas en los que se publicará el contenido",
    en = u"Languages to publish the content in"
)

translations.define("FacebookPublicationForm.publication_targets",
    ca = u"Canals on es publicarà el contingut",
    es = u"Canales donde se publicará el contenido",
    en = u"Channels where the content will be published"
)

translations.define("FacebookPublicationForm.languages",
    ca = u"Idiomes",
    es = u"Idiomas",
    en = u"Languages"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "section_title",
    ca = u"Publicar a Facebook",
    es = u"Publicar en Facebook",
    en = u"Publish with Facebook"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "publication_form.submit_button",
    ca = u"Publicar",
    es = u"Publicar",
    en = u"Publish"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "publication_form.check_button",
    ca = u"Comprovar",
    es = u"Comprobar",
    en = u"Check"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "selection_block",
    ca = u"Contingut a publicar:",
    es = u"Contenido a publicar:",
    en = u"Content to publish:"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "success",
    ca = u"Publicat amb èxit",
    es = u"Publicado con éxito",
    en = u"Published successfully"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "already_published",
    ca = u"Ja publicat",
    es = u"Ya publicado",
    en = u"Already published"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "not_yet_published",
    ca = u"Encara no s'ha publicat",
    es = u"Todavía no se ha publicado",
    en = u"Not published yet"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "publishable_header",
    ca = u"Element",
    es = u"Elemento",
    en = u"Item"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "target_header",
    ca = u"Destí",
    es = u"Destino",
    en = u"Destination"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "language_header",
    ca = u"Idioma",
    es = u"Idioma",
    en = u"Language"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "outcome_header",
    ca = u"Resultat",
    es = u"Resultado",
    en = u"Result"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookAlbumsView."
    "section_title",
    ca = u"Publicar fotos a Facebook",
    es = u"Publicar fotos en Facebook",
    en = u"Publish photos in Facebook"
)

translations.define("FacebookAlbumsForm.album_title",
    ca = u"Títol de l'àlbum",
    es = u"Título del álbum",
    en = u"Album title"
)

translations.define("FacebookAlbumsForm.album_description",
    ca = u"Descripció de l'àlbum",
    es = u"Descripción del álbum",
    en = u"Album description"
)

translations.define("FacebookAlbumsForm.subset",
    ca = u"Imatges incloses a l'àlbum",
    es = u"Imágenes incluídas en el álbum",
    en = u"Album photos"
)

translations.define("FacebookAlbumsForm.photo_languages",
    ca = u"Idiomes per les descripcions de les fotos",
    es = u"Idiomas para las descripciones de las fotos",
    en = u"Languages that photo captions are translated into"
)

translations.define("FacebookAlbumsForm.generate_story",
    ca = u"Anunciar l'àlbum al mur",
    es = u"Anunciar el álbum en el muro",
    en = u"Post a link to the new album in the wall"
)

translations.define("FacebookAlbumsForm.publication_targets",
    ca = u"Canals on es publicaran els àlbums",
    es = u"Canales donde se publicarán los álbumes",
    en = u"Channels where the albums will be published"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookAlbumsView."
    "publication_form.submit_button",
    ca = u"Publicar",
    es = u"Publicar",
    en = u"Publish"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookAlbumsView."
    "selection_block",
    ca = u"Àlbums a publicar:",
    es = u"Álbumes a publicar:",
    en = u"Albums to publish:"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookAlbumsView."
    "success",
    ca = u"Publicat amb èxit",
    es = u"Publicado con éxito",
    en = u"Published successfully"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookAlbumsView."
    "target_header",
    ca = u"Destí",
    es = u"Destino",
    en = u"Destination"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookAlbumsView."
    "outcome_header",
    ca = u"Resultat",
    es = u"Resultado",
    en = u"Result"
)

translations.define(
    "woost.extensions.facebookpublication.facebookpublicationtarget."
    "FacebookPublicationError-instance",
    ca = lambda instance: u"Error de publicació: <pre>%s</pre>"
                          % instance.response,
    es = lambda instance: u"Error de publicación: <pre>%s</pre>"
                          % instance.response,
    en = lambda instance: u"Publication error: <pre>%s</pre>"
                          % instance.response
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "publish_again_link",
    ca = u"Tornar a publicar",
    es = u"Volver a publicar",
    en = u"Publish again"
)

translations.define(
    "woost.extensions.facebookpublication.FacebookPublicationView."
    "loading_sign",
    ca = u"Enviant la petició a Facebook...",
    es = u"Enviando la petición a Facebook...",
    en = u"Sending the request to Facebook..."
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
from cocktail.modeling import cached_getter
from cocktail.schema import Collection, String, DictAccessor
from woost.controllers.backoffice.editcontroller import EditController
from woost.controllers.backoffice.useractions import get_user_action


class DifferencesController(EditController):

    view_class = "woost.views.BackOfficeDiffView"
    section = "diff"

    @cached_getter
    def source_item(self):
        item = self.stack_node.item
        return item.draft_source or item

    @cached_getter
    def output(self):
        output = EditController.output(self)
        output.update(
            submitted = False,
            source = self.source_item,
            target = self.stack_node.form_data,
            selected_action = get_user_action("diff")
        )
        return output




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
from cocktail import schema
from woost.models.item import Item


class Controller(Item):

    members_order = [
        "title",
        "python_name",
        "published_items"
    ]

    title = schema.String(
        unique = True,
        required = True,
        descriptive = True,
        translated = True
    )

    python_name = schema.String(
        required = True,
        text_search = False
    )

    published_items = schema.Collection(
        items = "woost.models.Publishable",
        bidirectional = True,
        editable = False
    )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
from __future__ import with_statement
import cherrypy
from ZODB.POSException import ConflictError
from cocktail.modeling import cached_getter
from cocktail.events import event_handler, when
from cocktail.schema import (
    Adapter, ErrorList, DictAccessor, Collection, Reference
)
from cocktail.translations import translations
from cocktail.persistence import datastore
from woost.models import (
    Site,
    Language,
    changeset_context,
    ChangeSet,
    get_current_user,
    restricted_modification_context,
    delete_validating,
    ReadTranslationPermission,
    ConfirmDraftPermission
)
from woost.controllers.notifications import notify_user, pop_user_notifications
from woost.controllers.backoffice.editstack import RelationNode, EditNode
from woost.controllers.backoffice.basebackofficecontroller \
        import BaseBackOfficeController


class EditController(BaseBackOfficeController):

    MAX_TRANSACTION_ATTEMPTS = 3
    section = None

    @event_handler
    def handle_processed(cls, event):
        controller = event.source
        controller.context["parent_handler"].section_redirection()
        controller.stack_node.section = controller.section

    @cached_getter
    def errors(self):
        if self.action:
            return ErrorList(self.action.get_errors(self, self.action_content))
        else:
            return []

    @cached_getter
    def available_languages(self):
        user = get_current_user()
        return [language
                for language in Language.codes
                if user.has_permission(
                    ReadTranslationPermission,
                    language = language
                )]

    @cached_getter
    def action(self):
        return self._get_user_action("item_action")

    @cached_getter
    def action_content(self):
        return [self.stack_node.item]

    @cached_getter
    def submitted(self):
        return self.action is not None

    @cached_getter
    def ready(self):
        return self.submitted and not self.errors

    def submit(self):
        self._invoke_user_action(self.action, self.action_content)

    def save_item(self, make_draft = False):

        for i in range(self.MAX_TRANSACTION_ATTEMPTS):
            user = get_current_user()
            stack_node = self.stack_node
            item = stack_node.item
            is_new = not item.is_inserted

            # Create a draft
            if make_draft:

                # From scratch
                if is_new:
                    item.is_draft = True

                # From an existing element
                else:
                    item = item.make_draft()

                item.author = user
                item.owner = user

            changeset = None

            with restricted_modification_context(
                item,
                user,
                member_subset = set(stack_node.form_schema.members())
            ):
                # Store the changes on a draft; this skips revision control
                if item.is_draft:
                    self._apply_changes(item)

                # Operate directly on a production item
                else:
                    with changeset_context(author = user) as changeset:
                        self._apply_changes(item)
            try:
                datastore.commit()
            except ConflictError:
                datastore.abort()
                datastore.sync()
            else:
                break

        change = changeset.changes.get(item.id) if changeset else None

        # Edit stack event
        stack_node.committed(
            user = user,
            changeset = changeset
        )

        # Application-wide event
        if not item.is_draft:
            if change is not None:
                self.context["cms"].item_saved(
                    item = item,
                    user = user,
                    is_new = is_new,
                    change = change
                )

        # User notification
        stack_node.item_saved_notification(is_new, change)

        # A new item or draft was created
        if is_new or make_draft:

            # The edit operation was the root of the edit stack; redirect the
            # browser to the new item
            if len(self.edit_stack) == 1:
                params = {"edit_stack": None} if make_draft else {}
                raise cherrypy.HTTPRedirect(
                    self.edit_uri(item, **params)
                )

            # The edit operation was nested; relate the created item to its
            # owner, and redirect the browser to the owner
            elif isinstance(stack_node.parent_node, RelationNode):
                member = stack_node.parent_node.member
                parent_edit_node = stack_node.get_ancestor_node(EditNode)
                parent_edit_node.relate(member, item)
                self.edit_stack.go(-3)

        # The user had arrived to the edit interface using a frontend link,
        # and has just saved the item at the top of the stack; redirect the
        # browser to the original frontend location. Notifications are
        # discarded before redirecting, since there is no guarantee that the
        # frontend will display them; this is clearly not ideal, but the
        # alternative (having them stack up and show all at once whenever the
        # user opens the backoffice) is not that great either.
        if stack_node.parent_node is None and self.edit_stack.root_url:
            pop_user_notifications()
            self.edit_stack.go_back()

    def confirm_draft(self):

        draft = self.stack_node.item
        target_item = draft.draft_source or draft
        is_new = draft is target_item
        user = get_current_user()

        user.require_permission(ConfirmDraftPermission, target = draft)
        member_subset = set(self.stack_node.form_schema.members())

        for i in range(self.MAX_TRANSACTION_ATTEMPTS):

            # Update the draft
            with restricted_modification_context(
                draft,
                user,
                member_subset = member_subset
            ):
                self._apply_changes(draft)

            # Confirm the draft
            with changeset_context(author = user) as changeset:
                with restricted_modification_context(
                    target_item,
                    user,
                    member_subset = member_subset
                ):
                    draft.confirm_draft()
            try:
                datastore.commit()
            except ConflictError:
                datastore.abort()
                datastore.sync()
            else:
                break

        # Edit stack event
        self.stack_node.committed(
            user = user,
            changeset = changeset
        )

        # Application-wide event
        self.context["cms"].item_saved(
            item = target_item,
            user = user,
            is_new = is_new,
            change = changeset.changes.get(target_item.id)
        )

        # User notification
        notify_user(
            translations(
                "woost.views.BackOfficeEditView Draft confirmed",
                item = target_item,
                is_new = is_new
            ),
            "success"
        )

        # Redirect back to the source item
        if not is_new:
            raise cherrypy.HTTPRedirect(
                self.edit_uri(target_item, edit_stack = None)
            )

    def _apply_changes(self, item):

        # Remove those instances that have been dettached from an integral
        # reference
        @when(item.changed)
        def delete_replaced_integral_children(event):
            if isinstance(event.member, Reference) \
            and event.member.integral \
            and event.previous_value is not None:
                delete_validating(event.previous_value)

        try:
            stack_node = self.stack_node
            stack_node.import_form_data(stack_node.form_data, item)
            item.insert()
            stack_node.saving(
                user = get_current_user(),
                changeset = ChangeSet.current
            )
        finally:
            item.changed.remove(delete_replaced_integral_children)

    @cached_getter
    def output(self):
        output = BaseBackOfficeController.output(self)
        stack_node = self.stack_node
        output.update(
            collections = self.context["parent_handler"].collections,
            edited_item = stack_node.item,
            edited_content_type = stack_node.content_type,
            errors = self.errors,
            form_schema = stack_node.form_schema,
            form_data = stack_node.form_data,
            changes = set(stack_node.iter_changes()),
            translations = stack_node.translations,
            section = self.section
        )
        return output




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
import re
from os.path import splitext
from cocktail.modeling import abstractmethod
from cocktail import schema
from cocktail.translations import translations
from woost.models.item import Item
from woost.models.publishable import Publishable
from woost.models.file import File


class PublicationScheme(Item):
    """Base class for all kinds of content publication schemes."""

    visible_from_root = False
    instantiable = False
    permanent_links = False

    site = schema.Reference(
        type = "woost.models.Site",
        bidirectional = True,
        visible = False
    )

    @abstractmethod
    def resolve_path(self, path):
        """Determines the publishable item that matches the indicated path.

        @param path: The path to evaluate; A list-like object describing a
            a path relative to the application's root.
        @type path: str list

        @return: A structure containing the matching item and its publication
            details. If no matching item can be found, None is returned
            instead.
        @rtype: L{PathResolution}
        """

    @abstractmethod
    def get_path(self, publishable, language):
        """Determines the canonical path of the indicated item.

        @param publishable: The item to get the canonical path for.
        @type publishable: L{Publishable<woost.models.publishable.Publishable>}

        @param language: The language to get the path in. Some schemes produce
            different canonical paths for the same content in different
            languages.
        @type language: str

        @return: The publication path for the indicated item, relative to the
            application's root. If none of the site's publishing schemes can
            produce a suitable path for the item, None is returned instead.
        @rtype: unicode
        """


class PathResolution(object):
    """A structure that provides publication information for an item.

    The L{PublicationScheme.resolve_path} and L{Site.resolve_path} methods use
    this class to wrap their return values.

    @var scheme: The publishing scheme used to resolve the publication details
        for the indicated path. This will only be set when calling
        L{Site.resolve_path}, to enable the caller to discern which of all the
        registered schemes handled the path's resolution.
    @type scheme: L{PublicationScheme}

    @var item: The publishable item that matches the indicated path.
    @type item: L{Publishable<woost.models.publishable.Publishable>}

    @var matching_path: The part of the evaluated path that was used by the
        publishing scheme to determine the matching item.
    @type matching_path: list

    @var extra_path: Additional path components that were not used by the
        publishing scheme when determining the path's matching item. This
        will be relayed to the item's handler when invoking the dispatching
        machinery.
    @type extra_path: list
    """
    scheme = None
    item = None
    matching_path = []
    extra_path = []

    def __init__(self,
        scheme,
        item,
        matching_path = None,
        extra_path = None):

        self.scheme = scheme
        self.item = item
        self.matching_path = matching_path if matching_path is not None else []
        self.extra_path = extra_path if extra_path is not None else []


class HierarchicalPublicationScheme(PublicationScheme):
    """A publication scheme that publishes pages following a name hierarchy.

    The name hierarchy is defined by the L{path<woost.models.Page.path>},
    and L{parent<woost.models.publishable.Publishable.parent>} members of the
    L{Publishable<woost.models.publishable.Publishable>} class.
    """
    instantiable = True

    def resolve_path(self, path):

        remaining_path = list(path)
        extra_path = []

        while remaining_path:
            page = Publishable.get_instance(
                full_path = u"/".join(remaining_path)
            )
            if page:
                return PathResolution(
                    self,
                    page,
                    remaining_path,
                    extra_path
                )
            extra_path.append(remaining_path.pop())

    def get_path(self, publishable, language):
        return publishable.full_path


class IdPublicationScheme(PublicationScheme):
    """A publication scheme that publishes items based on their id."""

    instantiable = True
    permanent_links = True

    def resolve_path(self, path):
        if path:
            try:
                id = int(path[0])
            except:
                pass
            else:
                publishable = Publishable.get_instance(id)
                if publishable is not None:
                    return PathResolution(
                        self,
                        publishable,
                        [path[0]],
                        path[1:]
                    )

    def get_path(self, publishable, language):
        return str(publishable.id)


class DescriptiveIdPublicationScheme(PublicationScheme):
    """A publication scheme that combines a unique identifier and a descriptive
    text fragment.

    @ivar word_separator: The character used to separate words on an item's
        title when crafting its descriptive fragment.
    @type word_separator: str

    @ivar id_regexp: The regular expression used to extract the unique
        identifier from an URI. The expression must define an 'id' named group.
    @type id_regexp: regular expression

    @param format: A python formatting string, used for composing an item's
        URI from its unique identifier and flattened title. Receives 'title'
        and 'id' parameters.
    @type format: str
    """
    instantiable = True
    permanent_links = True

    members_order = [
        "id_separator",
        "word_separator",
        "id_regexp",
        "title_splitter_regexp",
        "format",
        "include_file_extensions"
    ]

    id_separator = schema.String(
        required = True,
        default = "_",
        text_search = False
    )

    word_separator = schema.String(
        required = True,
        default = "-",
        text_search = False
    )

    id_regexp = schema.Member(
        required = True,
        default = schema.DynamicDefault(
            lambda: re.compile(r"(.+_)?(?P<id>\d+)(?P<ext>\.[a-zA-Z0-9]+)?$")
        ),
        normalization = lambda value:
            re.compile(value) if isinstance(value, basestring) else value,
        serialize_request_value = lambda value:
            value.pattern if value else value
    )

    title_splitter_regexp = schema.Member(
        required = True,
        default = schema.DynamicDefault(
            lambda: re.compile(r"\W+", re.UNICODE)
        ),
        normalization = lambda value:
            re.compile(value, re.UNICODE)
            if isinstance(value, basestring)
            else value,
        serialize_request_value = lambda value:
            value.pattern if value else value
    )

    format = schema.String(
        required = True,
        default = "%(title)s%(separator)s%(id)d",
        text_search = False
    )

    include_file_extensions = schema.Boolean(
        required = True,
        default = True
    )

    _uri_encodings = ["utf-8", "iso-8859-1"]

    def resolve_path(self, path):

        if path:

            ref = path[0]

            # Try to decode the supplied URI using a selection of different
            # string encodings
            if not isinstance(ref, unicode):
                for encoding in self._uri_encodings:
                    try:
                        ref = ref.decode(encoding)
                    except UnicodeDecodeError, ex:
                        pass
                    else:
                        break
                else:
                    raise ex

            # Discard descriptive text
            match = self.id_regexp.match(ref)

            if match is None:
                return None
            else:
                id = match.group("id")

            try:
                id = int(id)
            except:
                return None

            publishable = Publishable.get_instance(id)

            if publishable is not None:

                # A file extension was provided, but either the scheme doesn't
                # accept them, or the selected item doesn't match the requested
                # file type: 404
                try:
                    ext = match.group("ext")
                except:
                    ext = None

                if ext and (
                    not self.include_file_extensions
                    or ext != self.get_publishable_file_extension(publishable)
                ):
                    return None

                return PathResolution(
                    self,
                    publishable,
                    [path[0]],
                    path[1:]
                )

    def get_path(self, publishable, language):

        title = translations(publishable, language)

        if title:
            title = self.title_splitter_regexp.sub(
                self.word_separator,
                title
            )
            title = title.lower()
            ref = self.format % {
                "title": title,
                "id": publishable.id,
                "separator": self.id_separator
            }
        else:
            ref = str(publishable.id)

        if self.include_file_extensions:
            ext = self.get_publishable_file_extension(publishable)
            if ext:
                ref += ext

        return ref

    def get_publishable_file_extension(self, publishable):
        return getattr(publishable, "file_extension", None)




#-*- coding: utf-8 -*-
u"""Declares a user action for granting access to a Twitter account.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from oauth2 import Client, Consumer, Token
import cherrypy
from urllib import urlopen, quote_plus
from urlparse import parse_qsl
from simplejson import loads
from cocktail.persistence import datastore
from cocktail.translations import translations
from cocktail.controllers import Location, session
from woost.controllers.notifications import notify_user
from woost.controllers.backoffice.useractions import UserAction
from woost.extensions.twitterpublication.exceptions import TwitterAPIError
from woost.extensions.twitterpublication.twitterpublicationtarget \
    import TwitterPublicationTarget


class TwitterAuthUserAction(UserAction):

    content_type = TwitterPublicationTarget

    included = frozenset(["item_buttons"])
    excluded = frozenset(["new_item"])
    min = 1
    max = 1

    def invoke(self, controller, selection):

        target = selection[0]
        consumer = Consumer(target.app_id, target.app_secret)

        def request(client, *args, **kwargs):
            response, body = client.request(*args, **kwargs)
            if response["status"] != "200":
                raise TwitterAPIError(body)
            return body

        try:
            oauth_token = cherrypy.request.params.get("oauth_token")
            oauth_verifier = cherrypy.request.params.get("oauth_verifier")
            oauth_secret_session_key = "twitter_auth.%s.oauth_secret" % self.id

            if not oauth_token or not oauth_verifier:
                # Obtain a request token
                client = Client(consumer)

                location = Location.get_current(relative = False)
                location.query_string["item_action"] = self.id
                callback = quote_plus(str(location))

                body = request(
                    client,
                    "https://api.twitter.com/oauth/request_token"
                    "?oauth_callback=" + callback,
                    "GET"
                )
                data = dict(parse_qsl(body))
                session[oauth_secret_session_key] = data["oauth_token_secret"]

                # Redirect the user to the login form
                auth_url = (
                    "https://api.twitter.com/oauth/authorize?"
                    "oauth_token=%s"
                    "&oauth_callback=%s"
                    % (data["oauth_token"], callback)
                )
                raise cherrypy.HTTPRedirect(auth_url)
            else:
                token = Token(oauth_token, session[oauth_secret_session_key])
                token.set_verifier(oauth_verifier)
                client = Client(consumer, token)
                body = request(
                    client,
                    'https://api.twitter.com/oauth/access_token',
                    "POST"
                )
                data = dict(parse_qsl(body))
                target.auth_token = data["oauth_token"]
                target.auth_secret = data["oauth_token_secret"]
                datastore.commit()

        except TwitterAPIError, ex:
            notify_user(
                translations(ex),
                category = "error",
                transient = False
            )
        else:
            notify_user(
                translations(
                    "woost.extensions.twitterpublication."
                    "successful_authorization_notice"
                ),
                category = "success"
            )

TwitterAuthUserAction("twitter_auth").register()




#-*- coding: utf-8 -*-
"""Provides a variety of models to implement customizable discounts, taxes and
shipping costs.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import re
from decimal import Decimal
from datetime import datetime
from cocktail.modeling import abstractmethod
from cocktail.translations import translations
from cocktail import schema
from woost.models import Item, Role
from woost.extensions.locations.location import Location
from woost.extensions.ecommerce.ecommerceproduct import ECommerceProduct

override_regexp = re.compile(r"^=\s*((-|\+)?\d+(.\d+)?)$")
add_regexp = re.compile(r"^((-|\+)?\d+(.\d+)?)$")
override_percentage_regexp = re.compile(r"^=\s*((-|\+)?\d+(\.\d+)?)%$")
add_percentage_regexp = re.compile(r"^((-|\+)?\d+(\.\d+)?)%$")
free_units_regexp = re.compile(r"^\d\s*x\s*\d?$")


class ECommerceBillingConcept(Item):

    members_order = [
        "title",
        "enabled",
        "start_date",
        "end_date",
        "hidden",
        "scope",
        "eligible_countries",
        "eligible_products",
        "eligible_roles",
        "condition",
        "implementation"
    ]

    visible_from_root = False

    title = schema.String(
        translated = True,
        descriptive = True,
        required = True
    )

    enabled = schema.Boolean(
        required = True,
        default = True
    )

    start_date = schema.DateTime(
        indexed = True
    )

    end_date = schema.DateTime(
        indexed = True,
        min = start_date
    )

    hidden = schema.Boolean(
        default = False,
        required = True
    )

    scope = schema.String(
        required = True,
        enumeration = ["order", "purchase"],
        default = "order",
        edit_control = "cocktail.html.RadioSelector",
        translate_value = lambda value, language = None, **kwargs:
            "" if not value
               else translations(
                        "ECommerceBillingConcept.scope-" + value,
                        language,
                        **kwargs)
    )

    condition = schema.CodeBlock(
        language = "python"
    )

    eligible_countries = schema.Collection(
        items = schema.Reference(
            type = Location,
            relation_constraints = [Location.location_type.equal("country")]
        ),
        related_end = schema.Collection()
    )

    eligible_products = schema.Collection(
        items = schema.Reference(type = ECommerceProduct),
        related_end = schema.Collection()
    )

    eligible_roles = schema.Collection(
        items = schema.Reference(type = Role),
        related_end = schema.Collection()
    )

    implementation = schema.CodeBlock(
        language = "python"
    )

    def is_current(self):
        return (self.start_date is None or self.start_date <= datetime.now()) \
           and (self.end_date is None or self.end_date > datetime.now())

    def applies_to(self, item, costs = None):

        if not self.enabled:
            return False

        if not self.is_current():
            return False

        from woost.extensions.ecommerce.ecommerceproduct \
            import ECommerceProduct

        order = None
        purchase = None
        product = None

        if isinstance(item, ECommerceProduct):

            if self.eligible_products and item not in self.eligible_products:
                return False

            product = item

        elif self.scope == "order":
            from woost.extensions.ecommerce.ecommerceorder \
                import ECommerceOrder
            if not isinstance(item, ECommerceOrder):
                return False

            if self.eligible_products and not any(
                purchase.product in self.eligible_products
                for purchase in item.purchases
            ):
                return False

            order = item

        elif self.scope == "purchase":
            from woost.extensions.ecommerce.ecommercepurchase \
                import ECommercePurchase
            if not isinstance(item, ECommercePurchase):
                return False

            if self.eligible_products \
            and item.product not in self.eligible_products:
                return False

            order = item.order
            purchase = item
            product = item.product

        if self.eligible_countries and (
            order is None
            or order.country is None
            or not any(
                order.country.descends_from(region)
                for region in self.eligible_countries
            )
        ):
            return False

        # Eligible roles
        if self.eligible_roles and (
            order is None
            or order.customer is None
            or not any(
                role in self.eligible_roles
                for role in order.customer.iter_roles()
            )
        ):
            return False

        # Custom condition
        if self.condition:
            context = {
                "self": self,
                "order": order,
                "purchase": purchase,
                "product": product,
                "costs": costs,
                "applies": True
            }
            exec self.condition in context
            if not context["applies"]:
                return False

        return True

    def apply(self, item, costs):

        costs["concepts"].append(self)

        kind, value = self.parse_implementation()

        if kind == "override":
            applicable_concepts = []
            for concept in costs["concepts"]:
                concept_kind, concept_value = concept.parse_implementation()
                if concept_kind not in ("add", "override") or concept is self:
                    applicable_concepts.append(concept)

            costs["concepts"] = applicable_concepts
            costs["cost"] = value

        elif kind == "add":
            costs["cost"] += value

        elif kind == "override_percentage":
            applicable_concepts = []
            for concept in costs["concepts"]:
                concept_kind, concept_value = concept.parse_implementation()
                if concept_kind not in ("add_percentage", "override_percentage") or concept is self:
                    applicable_concepts.append(concept)

            costs["concepts"] = applicable_concepts
            costs["percentage"] = value

        elif kind == "add_percentage":
            costs["percentage"] += value

        elif kind == "free_units":
            delivered, paid = value
            q, r = divmod(item.quantity, delivered)
            costs["paid_units"] = q * paid + r

        elif kind == "custom":
            context = {"self": self, "item": item, "costs": costs}
            exec value in context

    def parse_implementation(self):

        value = self.implementation

        # Cost override
        match = override_regexp.match(value)
        if match:
            return ("override", Decimal(match.group(1)))

        # Additive cost
        match = add_regexp.match(value)
        if match:
            return ("add", Decimal(match.group(1)))

        # Override percentage
        match = override_percentage_regexp.match(value)
        if match:
            return ("override_percentage", Decimal(match.group(1)))

        # Override percentage
        match = add_percentage_regexp.match(value)
        if match:
            return ("add_percentage", Decimal(match.group(1)))

        # Free units discount ("3x2", "2x1", etc)
        match = free_units_regexp.match(value)
        if match:
            return ("free_units", (int(match.group(1)), int(match.group(2))))

        return ("custom", value)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from tpv.pasat4b import Pasat4bPaymentGateway as Implementation
from cocktail import schema
from cocktail.translations import translations
from woost.extensions.payments.paymentgateway import PaymentGateway


class Pasat4bPaymentGateway(PaymentGateway, Implementation):

    instantiable = True

    default_label = schema.DynamicDefault(
        lambda: translations("Pasat4bPaymentGateway.label default")
    )

    merchant_code = schema.String(
        required = True,
        shadows_attribute = True,
        text_search = False
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from __future__ import with_statement
from cocktail.modeling import cached_getter
from woost.controllers.backoffice.editstack import EditNode
from woost.extensions.usermodels import models_access


class UserMemberEditNode(EditNode):

    @cached_getter
    def form_adapter(self):
        form_adapter = EditNode.form_adapter(self)

        if not self.content_type.edit_controls:
            form_adapter.exclude(["member_edit_control"])

        if not self.content_type.search_controls:
            form_adapter.exclude(["member_search_control"])

        return form_adapter

    @cached_getter
    def form_schema(self):
        form_schema = EditNode.form_schema(self)

        edit_control = form_schema.get_member("member_edit_control")
        if edit_control:
            edit_control.enumeration = self.item.edit_controls

        search_control = form_schema.get_member("member_search_control")
        if search_control:
            search_control.enumeration = self.item.search_controls

        return form_schema

    def import_form_data(self, form_data, item):
        # Overriden to require exclusive access to application schemas. The
        # update will wait for all current requests to end, and then will block
        # any further incoming requests until the schemas have been modified.

        # The current request doesn't count when waiting for exclusive access
        # to the models:
        models_access.leave()

        # Require exclusive access
        try:
            with models_access.locking:
                return EditNode.import_form_data(self, form_data, item)

        # Claim usage of the models once again
        finally:
            models_access.enter()




#-*- coding: utf-8 -*-
u"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			September 2010
"""
import smtplib
import cherrypy
from simplejson import dumps
from cocktail import schema
from cocktail.events import event_handler
from cocktail.modeling import cached_getter
from cocktail.translations import set_language
from cocktail.controllers.location import Location
from woost.models import Site, User, get_current_user
from woost.models.permission import ReadPermission
from woost.controllers.backoffice.editcontroller import EditController
from woost.extensions.mailer.mailing import Mailing, MAILING_STARTED, MAILING_FINISHED, tasks
from woost.extensions.mailer.sendemailpermission import SendEmailPermission


class SendEmailController(EditController):

    view_class = "woost.extensions.mailer.SendEmailView"
    section = "send_email"
    task_id = None

    @event_handler
    def handle_traversed(cls, event):
        user = get_current_user()
        user.require_permission(SendEmailPermission)

    @cached_getter
    def smtp_server(self):
        site = Site.main
        smtp = smtplib.SMTP(site.smtp_host, smtplib.SMTP_PORT)

        if site.smtp_user and site.smtp_password:
            smtp.login(
                str(site.smtp_user),
                str(site.smtp_password)
            )

        return smtp

    @cached_getter
    def submitted(self):
        return self.action is not None \
            or self.mailer_action in ("test", "send")

    @cached_getter
    def mailer_action(self):
        mailing = self.context["cms_item"]
        mailer_action = self.params.read(schema.String("mailer_action"))

        if mailer_action in ("test", "send") and mailing.status == MAILING_FINISHED:
            raise cherrypy.HTTPError(403, "Forbidden")

        if mailer_action is None:
            if mailing.status is None:
                mailer_action = "confirmation"
            elif mailing.status == MAILING_STARTED:
                mailer_action = "send"
            elif mailing.status == MAILING_FINISHED:
                mailer_action = "show"

        # A mailer action is required
        assert mailer_action

        return mailer_action

    @cherrypy.expose
    def mailing_state(self, task_id, *args, **kwargs):
        cherrypy.response.headers["Content-Type"] = "application/json"

        task = tasks.get(int(task_id))

        # If task is expired show the status from the mailing object
        if task is None:
            mailing = self.context["cms_item"]
        else:
            mailing = Mailing.get_instance(task.mailing_id)

        if mailing is None:
            return dumps({})

        user = get_current_user()
        if not user.has_permission(ReadPermission, target = mailing):
            raise cherrypy.HTTPError(403, "Forbidden")

        tasks.remove_expired_tasks()

        return dumps({
            "sent": len(mailing.sent),
            "errors": len(mailing.errors),
            "total": mailing.total,
            "completed": task and task.completed or True
        })

    def submit(self):

        mailing = self.context["cms_item"]

        if self.mailer_action == "test":

            # Send a test email
            mailing._v_template_values = {
                "cms": self.context["cms"],
                "base_url": unicode(Location.get_current_host()).rstrip("/")
            }
            test_email = self.params.read(schema.String("test_email"))
            # Create a fake user
            receiver = User(email = test_email)
            set_language(mailing.language.iso_code)
            mailing.send_message(self.smtp_server, receiver)

        elif self.mailer_action == "send":

            # Send the mailing
            self.task_id = mailing.id
            template_values = {
                "cms": self.context["cms"],
                "base_url": unicode(Location.get_current_host()).rstrip("/")
            }
            mailing.send(self.smtp_server, template_values, self.context.copy())

        else:
            EditController.submit(self)

    @cached_getter
    def output(self):
        output = EditController.output(self)

        if self.task_id:
            task_id = self.task_id
        else:
            task_id = self.context["cms_item"] and self.context["cms_item"].id or None

        output.update(
            action = self.mailer_action,
            task_id = task_id
        )
        return output




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations
from cocktail.persistence import datastore
from cocktail.html import templates
from woost.models import Extension


translations.define("BlocksExtension",
    ca = u"Blocs de contingut",
    es = u"Bloques de contenido",
    en = u"Content blocks"
)

translations.define("BlocksExtension-plural",
    ca = u"Blocs de contingut",
    es = u"Bloques de contenido",
    en = u"Content blocks"
)


class BlocksExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet la creació de contingut utilitzant blocs""",
            "ca"
        )
        self.set("description",
            u"""Permite la creación de contenido usando bloques""",
            "es"
        )
        self.set("description",
            u"""Allows the creation of content using blocs""",
            "en"
        )

    def _load(self):

        from woost.extensions.blocks import (
            strings,
            block,
            containerblock,
            slideshowblock,
            imagegalleryblock,
            bannerblock,
            menublock,
            richtextblock,
            translatedrichtextblock,
            twittertimelineblock,
            linksblock,
            folderblock,
            loginblock,
            iframeblock,
            blockspage,
            migration
        )

        from woost.extensions.vimeo import VimeoExtension

        if VimeoExtension.instance.enabled:
            from woost.extensions.blocks import vimeoblock

        # Install an overlay for the frontend edit panel
        templates.get_class("woost.extensions.blocks.EditPanelOverlay")

        self.install()

    def _install(self):
        from woost.models import Template, extension_translations

        self._create_asset(
            Template,
            "blocks_page_template",
            title = extension_translations,
            engine = "cocktail",
            identifier = "woost.extensions.blocks.BlocksPageView"
        )




#-*- coding: utf-8 -*-
u"""Defines the `ProductController` class.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail.translations import translations
from cocktail.controllers import request_property, Location
from cocktail.controllers.formprocessor import FormProcessor, Form
from woost.models import Publishable
from woost.controllers.notifications import notify_user
from woost.controllers.documentcontroller import DocumentController
from woost.extensions.ecommerce.ecommerceproduct import ECommerceProduct
from woost.extensions.ecommerce.ecommercepurchase import ECommercePurchase
from woost.extensions.ecommerce.basket import Basket


class ProductController(FormProcessor, DocumentController):

    class AddProductForm(Form):

        actions = ("add_product",)
        redirect_to_basket = False

        @request_property
        def product(self):
            return self.controller.context["publishable"]

        @request_property
        def model(self):
            return self.product.purchase_model

        @request_property
        def adapter(self):

            adapter = Form.adapter(self)
            adapter.implicit_copy = False

            for member in self.model.get_options():
                adapter.copy(member.name)

            return adapter

        def create_instance(self):
            instance = Form.create_instance(self)
            instance.product = self.product
            return instance

        def submit(self):
            Form.submit(self)
            Basket.get().add_purchase(self.instance)
            Basket.store()

            notify_user(
                translations(
                    "woost.extensions.ecommerce.product_added_notice",
                    product = self.product
                ),
                "product_added",
                transient = False
            )

            if self.redirect_to_basket:
                raise cherrypy.HTTPRedirect(
                    Publishable.require_instance(
                        qname = "woost.extensions.ecommerce.basket_page"
                    ).get_uri()
                )
            else:
                Location.get_current().go("GET")




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""

default_format = "JPEG"

# Mappings between MIME types and PIL formats
formats_by_mime_type = {
    "image/jpeg": "JPEG",
    "image/pjpeg": "JPEG",
    "image/png": "PNG",
    "image/x-png": "PNG",
    "image/gif": "GIF",
    "image/tiff": "TIFF"
}

mime_types_by_format = dict(
    (v, k)
    for k, v in formats_by_mime_type.iteritems()
)

formats_by_extension = {
    "jpg": "JPEG",
    "jpeg": "JPEG",
    "png": "PNG",
    "gif": "GIF",
    "tiff": "TIFF"
}

extensions_by_format = dict(
    (v, k)
    for k, v in formats_by_extension.iteritems()
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from pkg_resources import resource_filename
import cherrypy
from cocktail import schema
from cocktail.translations import translations
from woost.models import (
    Site,
    Extension,
    Role,
    Document,
    Controller,
    Template
)

translations.define("WebConsoleExtension",
    ca = u"Intèrpret de Python",
    es = u"Intérprete de Python",
    en = u"Python interpreter"
)

translations.define("WebConsoleExtension-plural",
    ca = u"Intèrpret de Python",
    es = u"Intérprete de Python",
    en = u"Python interpreter"
)

class WebConsoleExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Proporciona accés directe a l'intèrpret de Python en que
            s'executa l'aplicació a través de la web""",
            "ca"
        )
        self.set("description",
            u"""Proporciona acceso directo al intérprete de Python que ejecuta
            la aplicación a través de la web""",
            "es"
        )
        self.set("description",
            u"""Provides a web interface to access the Python interpreter that
            the application runs on""",
            "en"
        )

    def _load(self):

        from woost.extensions.webconsole import strings
        from woost.controllers.cmscontroller import CMSController

        CMSController.ApplicationContainer.webconsole_resources = \
            cherrypy.tools.staticdir.handler(
                section = "webconsole_resources",
                dir = resource_filename("webconsole", "resources")
            )

    def _install(self):

        from woost.models import extension_translations
        from woost.extensions.webconsole.webconsolepermission \
            import WebConsolePermission

        # Permissions: by default, only administrators can use the web
        # console
        administrators = Role.get_instance(qname = "woost.administrators")

        if administrators:
            web_console_permission = WebConsolePermission()
            web_console_permission.insert()
            administrators.permissions.append(web_console_permission)

        # Controller
        controller = self._create_asset(
            Controller,
            "controller",
            python_name = \
                "woost.extensions.webconsole.cmswebconsole.CMSWebConsole",
            title = extension_translations
        )

        # Template
        template = self._create_asset(
            Template,
            "template",
            engine = "cocktail",
            identifier = "webconsole.WebConsole",
            title = extension_translations
        )

        # Page
        page = self._create_asset(
            Document,
            "page",
            parent = Site.main.home,
            path = "webconsole",
            hidden = True,
            requires_https = True,
            controller = controller,
            template = template,
            per_language_publication = False,
            title = extension_translations
        )


def breakpoint(open_browser = False, stack_depth = 0):
    """Set a `~webconsole.utils.breakpoint` that is only triggered by users
    with `code execution rights <WebConsolePermission>`.
    """
    from cocktail.controllers import Location
    from webconsole.utils import breakpoint as webconsole_breakpoint
    from woost.models import get_current_user, Publishable, Site
    from woost.extensions.webconsole.webconsolepermission \
        import WebConsolePermission

    user = get_current_user()

    if user and user.has_permission(WebConsolePermission):

        def initializer(session):
            if open_browser:

                # Find the web console document
                webconsole = Publishable.require_instance(
                    qname = "woost.extensions.webconsole.page"
                )

                # Determine the URI for the breakpoint session
                webconsole_location = Location.get_current_host()
                webconsole_location.path_info = \
                    u"/" + Site.main.get_path(webconsole)
                webconsole_location.query_string["session_id"] = session.id

                # Open a web browser tab pointing at the URI
                from webbrowser import open
                open(str(webconsole_location))

        return webconsole_breakpoint(
            initializer = initializer,
            stack_depth = stack_depth + 1
        )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
from __future__ import with_statement
from ZODB.POSException import ConflictError
import cherrypy
from simplejson import dumps
from cocktail.modeling import cached_getter
from cocktail.events import event_handler
from cocktail.iteration import first
from cocktail.translations import translations
from cocktail.schema import Reference, String, Integer, Collection, Reference
from cocktail.persistence import datastore
from woost.models import Item, changeset_context, get_current_user
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController


class MoveController(BaseBackOfficeController):

    MAX_TRANSACTION_ATTEMPTS = 3

    @cached_getter
    def handling_ajax_request(self):
        return self.rendering_format == "json"

    @cached_getter
    def root(self):
        return self.params.read(Reference("root", type = Item))

    @cached_getter
    def member(self):
        key = self.params.read(String("member"))
        return self.item.__class__[key] if key and self.item else None

    @cached_getter
    def collection(self):
        return self.item.get(self.member)

    @cached_getter
    def selection(self):
        return self.params.read(
            Collection("selection", items = Reference(type = Item))
        )

    @cached_getter
    def slot(self):
        param = self.params.read(String("slot", format = r"\d+-\d+"))
        return map(int, param.split("-")) if param else None

    @cached_getter
    def item(self):
        return self.slot and Item.index[self.slot[0]]

    @cached_getter
    def position(self):
        return self.slot and self.slot[1]

    @cached_getter
    def action(self):
        return self.params.read(String("action"))

    @cached_getter
    def ready(self):
        return self.member and self.slot and self.selection

    def submit(self):

        collection = self.collection
        selection = self.selection
        parent = self.item
        position = self.position
        related_end = self.member.related_end

        size = len(collection)

        if position < 0:
            position = size + position

        position = min(position, size)

        if any(parent.descends_from(item) for item in selection):
            raise TreeCycleError()

        for i in range(self.MAX_TRANSACTION_ATTEMPTS):
            with changeset_context(get_current_user()):
                for item in reversed(selection):

                    if isinstance(related_end, Reference) \
                    and item.get(related_end) is parent:
                        collection.remove(item)

                    collection.insert(position, item)
            try:
                datastore.commit()
            except ConflictError:
                datastore.abort()
                datastore.sync()
            else:
                break

    def handle_error(self, error):
        if self.handling_ajax_request:
            self.output["error"] = translations(error)
        else:
            BaseBackOfficeController.handle_error(self, error)

    @event_handler
    def handle_after_request(cls, event):

        controller = event.source

        if not controller.handling_ajax_request:
            if controller.action == "cancel" or controller.successful:
                raise cherrypy.HTTPRedirect(controller.contextual_uri())

    view_class = "woost.views.BackOfficeMoveView"

    @cached_getter
    def output(self):
        if self.handling_ajax_request:
            return {}
        else:
            output = BaseBackOfficeController.output(self)
            output.update(
                root = self.root,
                item = self.item,
                position = self.position,
                selection = self.selection
            )
            return output


class TreeCycleError(Exception):
    """An exception raised when trying to move an element to a position that
    would turn the tree into a graph."""




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			August 2008
"""
from cocktail.translations import translations
from cocktail.html import Element, templates
from cocktail.controllers import context
from woost.models import Item

TreeView = templates.get_class("cocktail.html.TreeView")


class ContentTypeTree(TreeView):

    root = Item
    root_visible = True
    plural_labels = False
    icon_image_factory = "icon16.png"

    def create_label(self, content_type):

        label = TreeView.create_label(self, content_type)

        for schema in content_type.descend_inheritance(True):
            label.add_class(schema.name)

        img = Element("img")
        img["src"] = context["cms"].image_uri(
            content_type, self.icon_image_factory
        )
        img.add_class("icon")
        label.insert(0, img)

        return label

    def get_item_label(self, content_type):
        if self.plural_labels:
            return translations(content_type.__name__ + "-plural")
        else:
            return translations(content_type.__name__)

    def get_child_items(self, content_type):
        return sorted(
            content_type.derived_schemas(recursive = False),
            key = lambda ct: translations(ct.__name__)
        )

    def filter_item(self, content_type):
        return content_type.visible





#-*- coding: utf-8 -*-
import cherrypy
from woost.models.extension import load_extensions
import _PROJECT_MODULE_.models
import _PROJECT_MODULE_.views
from _PROJECT_MODULE_.controllers import _PROJECT_NAME_CMS

load_extensions()
cms = _PROJECT_NAME_CMS()
application = cms.mount()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
from cocktail.html import Element
from cocktail.translations import translations
from cocktail.controllers import context


class ItemLabel(Element):

    item = None
    image_factory = "backoffice_small_thumbnail"
    icon_visible = True
    thumbnail = True
    referer = None

    def _ready(self):
        Element._ready(self)

        if self.item:

            for schema in self.item.__class__.descend_inheritance(True):
                self.add_class(schema.name)

            if self.icon_visible:
                self.append(self.create_icon())

            self.append(self.get_label())

    def create_icon(self):
        img = Element("img")
        img.add_class("icon")
        get_image_uri = getattr(self.item, "get_image_uri", None)

        if get_image_uri:
            img["src"] = get_image_uri(self.image_factory)
        else:
            image_factory = self.image_factory or "default"

            if "." not in image_factory:
                from woost.models.rendering.formats import (
                    extensions_by_format,
                    default_format
                )
                extension = extensions_by_format[default_format]
                image_factory += "." + extension

            img["src"] = context["cms"].image_uri(self.item, image_factory)

        return img

    def get_label(self):
        return translations(self.item, referer = self.referer)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
from woost.models import Publishable
from woost.controllers.backoffice.useractions import UserAction
from woost.extensions.staticsite.staticsitedestination import \
    StaticSiteDestination
from woost.extensions.staticsite.exportationpermission import \
    ExportationPermission


class ExportStaticAction(UserAction):
    min = None
    max = None
    included = frozenset(["toolbar"])
    content_type = Publishable

    def is_permitted(self, user, target):
        return any([
            user.has_permission(
                ExportationPermission,
                destination = destination
            )
            for destination in StaticSiteDestination.select()
        ])

ExportStaticAction("export_static").register()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from cocktail.translations import translations
from cocktail.translations.helpers import ca_possessive
from woost.translations.strings import content_permission_translation_factory

translations.define(
    "woost.controllers.backoffice.useractions.TransitionAction state set",
    ca = lambda item: u"S'ha canviat l'estat de <em>%s</em> a <em>%s</em>"
        % (translations(item, "ca"), translations(item.workflow_state, "ca")),
    es = lambda item: u"Se ha cambiado el estado de <em>%s</em> a <em>%s</em>"
        % (translations(item, "es"), translations(item.workflow_state, "es")),
    en = lambda item: u"State of <em>%s</em> changed to <em>%s</em>"
        % (translations(item, "en"), translations(item.workflow_state, "en"))
)

# Item
#------------------------------------------------------------------------------
translations.define("Item.workflow_state",
    ca = u"Estat",
    es = u"Estado",
    en = u"State"
)

# TransitionPermission
#------------------------------------------------------------------------------
translations.define("TransitionPermission",
    ca = u"Permís de transició",
    es = u"Permiso de transición",
    en = u"Transition permission"
)

translations.define("TransitionPermission-plural",
    ca = u"Permisos de transició",
    es = u"Permisos de transición",
    en = u"Transition permissions"
)

translations.define("TransitionPermission.transition",
    ca = u"Transició",
    es = u"Transición",
    en = u"Transition"
)

translations.define(
    "woost.extensions.workflow.transitionpermission."
    "TransitionPermission-instance",
    ca = content_permission_translation_factory("ca",
        lambda permission, subject, **kwargs:
            "%s %s" % (
                translations(permission.transition, "ca", **kwargs),
                subject
            )
            if permission.transition
            else "canviar l'estat " + ca_possessive(subject),
    ),
    es = content_permission_translation_factory("es",
        lambda permission, subject, **kwargs:
            "%s %s" % (
                translations(permission.transition, "es", **kwargs),
                subject
            )
            if permission.transition
            else "cambiar el estado de " + subject,
    ),
    en = content_permission_translation_factory("en",
        lambda permission, subject, **kwargs:
            "%s %s" % (
                translations(permission.transition, "en", **kwargs),
                subject
            )
            if permission.transition
            else "change the state of " + subject
    )
)

# TransitionTrigger
#------------------------------------------------------------------------------
translations.define("TransitionTrigger",
    ca = u"Disparador de transició",
    es = u"Disparador de transición",
    en = u"Transition trigger"
)

translations.define("TransitionTrigger-plural",
    ca = u"Disparadors de transició",
    es = u"Disparadores de transición",
    en = u"Transition triggers"
)

translations.define("TransitionTrigger.transition",
    ca = u"Transició a observar",
    es = u"Transición a observar",
    en = u"Watched transition"
)

# State
#------------------------------------------------------------------------------
translations.define("State",
    ca = u"Estat",
    es = u"Estado",
    en = u"State"
)

translations.define("State-plural",
    ca = u"Estats",
    es = u"Estados",
    en = u"States"
)

translations.define("State.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("State.outgoing_transitions",
    ca = u"Transicions de sortida",
    es = u"Transiciones de salida",
    en = u"Outgoing transitions"
)

translations.define("State.incoming_transitions",
    ca = u"Transicions d'entrada",
    es = u"Transiciones de entrada",
    en = u"Incoming transitions"
)

# Transition
#------------------------------------------------------------------------------
translations.define("Transition",
    ca = u"Transició",
    es = u"Transición",
    en = u"Transition"
)

translations.define("Transition-plural",
    ca = u"Transicions",
    es = u"Transiciones",
    en = u"Transitions"
)

translations.define("Transition.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Title"
)

translations.define("Transition.source_state",
    ca = u"Estat origen",
    es = u"Estado origen",
    en = u"Source state"
)

translations.define("Transition.target_state",
    ca = u"Estat destí",
    es = u"Estado destino",
    en = u"Target state"
)

translations.define("Transition.transition_form",
    ca = u"Formulari de transició",
    es = u"Formulario de transición",
    en = u"Transition form"
)

translations.define("Transition.transition_permissions",
    ca = u"Permisos",
    es = u"Permisos",
    en = u"Permissions"
)

# SetStateTriggerResponse
#------------------------------------------------------------------------------
translations.define("SetStateTriggerResponse",
    ca = u"Canvi d'estat",
    es = u"Cambio de estado",
    en = u"State change"
)

translations.define("SetStateTriggerResponse-plural",
    ca = u"Canvis d'estat",
    es = u"Cambios de estado",
    en = u"State changes"
)

translations.define("SetStateTriggerResponse.target_state",
    ca = u"Estat de destí",
    es = u"Estado de destino",
    en = u"Destination state"
)

# TransitionAction
#------------------------------------------------------------------------------
translations.define("Action transition",
    ca = u"Transició",
    es = u"Transición",
    en = u"Transition"
)

# Views
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.workflow any transition",
    ca = u"Qualsevol",
    es = u"Cualquiera",
    en = u"Any"
)

translations.define(
    "woost.extensions.workflow.TransitionView cancel",
    ca = u"Cancel·lar",
    es = u"Cancelar",
    en = u"Cancel"
)

translations.define(
    "woost.extensions.workflow.TransitionView title",
    ca = lambda item, transition: u"%s l'element %s" % (
        transition.title, translations(item)
    ),
    es = lambda item, transition: u"%s el elemento %s" % (
        transition.title, translations(item)
    ),
    en = lambda item, transition: u"%s the %s item" % (
        transition.title, translations(item)
    )
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import cherrypy
from cocktail.modeling import getter
from cocktail.persistence import datastore
from cocktail.controllers import session, Location
from woost.models import User, set_current_user
from woost.controllers.module import Module


class AuthenticationModule(Module):

    SESSION_KEY = "user_id"

    identifier_field = User.email

    def process_request(self):

        set_current_user(
            self.get_user_from_session()
            or self.anonymous_user
        )

        self.process_login()
        self.process_logout()

    def process_login(self):
        params = cherrypy.request.params
        if "authenticate" in params:
            self.login(
                params.get("user"),
                params.get("password")
            )

            # Request the current location again, with all authentication
            # parameters stripped
            location = Location.get_current()
            for params in (location.query_string, location.form_data):
                params.pop("user", None)
                params.pop("password", None)
                params.pop("authenticate", None)
            location.go("GET")

    def process_logout(self):
        if "logout" in cherrypy.request.params:
            self.logout()

    @getter
    def anonymous_user(self):
        return User.get_instance(qname = "woost.anonymous_user")

    def get_user_from_session(self):
        session_user_id = session.get(self.SESSION_KEY)
        if session_user_id:
            return User.get_instance(session_user_id)

    def set_user_session(self, user):
        session[self.SESSION_KEY] = user.id
        set_current_user(user)

    def login(self, identifier, password):
        """Attempts to establish a new user session, using the given user
        credentials.

        @param identifier: An identifier matching a single user in the
            database. Matches are made against the field indicated by the
            L{identifier_field>} attribute.
        @type identifier: str

        @param password: The unencrypted password for the user.
        @type: str

        @return: The authenticated user.
        @rtype: L{User<woost.models.user.User>}

        @raise L{AuthenticationFailedError}: Raised if the provided user
            credentials are invalid.
        """
        identifier = identifier.strip()

        if identifier and password:
            params = {self.identifier_field.name: identifier}
            user = User.get_instance(**params)

            if user and user.enabled and user.test_password(password):
                self.set_user_session(user)
                return user

        raise AuthenticationFailedError(identifier)

    def logout(self):
        """Ends the current user session."""
        session.delete()
        set_current_user(self.anonymous_user)


class AuthenticationFailedError(Exception):
    """An exception raised when a user authentication attempt fails."""

    def __init__(self, identifier):
        Exception.__init__(self, "Wrong user credentials")
        self.identifier = identifier




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2008
"""
from datetime import datetime
from cocktail.modeling import getter, ListWrapper, SetWrapper
from cocktail.events import event_handler, Event
from cocktail import schema
from cocktail.translations import translations
from cocktail.persistence import (
    datastore,
    PersistentObject,
    PersistentClass,
    PersistentMapping,
    MaxValue
)
from woost.models.changesets import ChangeSet, Change
from woost.models.action import Action

# Extension property that allows changing the controller that handles a
# collection in the backoffice
schema.Collection.edit_controller = \
    "woost.controllers.backoffice.collectioncontroller." \
    "CollectionController"

# Extension property that makes it easier to customize the edit view for a
# collection in the backoffice
schema.Collection.edit_view = None

# Extension property that sets the default type that should be shown by default
# when opening an item selector for the indicated property
schema.RelationMember.selector_default_type = None

# Extension property that allows to indicate that specific members don't modify
# the 'last_update_time' member of items when changed
schema.Member.affects_last_update_time = True


class Item(PersistentObject):
    """Base class for all CMS items. Provides basic functionality such as
    authorship, group membership, draft copies and versioning.
    """
    members_order = [
        "id",
        "qname",
        "author",
        "owner"
    ]

    # Enable full text indexing for all items (although the Item class itself
    # doesn't provide any searchable text field by default, its subclasses may,
    # or it may be extended; by enabling full text indexing at the root class,
    # heterogeneous queries on the whole Item class will use available
    # indexes).
    full_text_indexed = True

    # Extension property that indicates if content types should be visible from
    # the backoffice root view
    visible_from_root = True

    # Extension property that indicates if the backoffice should show child
    # entries for this content type in the type selector
    collapsed_backoffice_menu = False

    def __translate__(self, language, **kwargs):
        if self.draft_source is not None:
            return translations(
                "woost.models.Item draft copy",
                language,
                item = self.draft_source,
                draft_id = self._draft_id,
                **kwargs
            )
        else:
            return PersistentObject.__translate__(self, language, **kwargs)

    # Unique qualified name
    #--------------------------------------------------------------------------
    qname = schema.String(
        unique = True,
        indexed = True,
        text_search = False,
        listed_by_default = False,
        member_group = "administration"
    )

    # Backoffice customization
    #--------------------------------------------------------------------------
    show_detail_view = "woost.views.BackOfficeShowDetailView"
    show_detail_controller = \
        "woost.controllers.backoffice.showdetailcontroller." \
        "ShowDetailController"
    collection_view = "woost.views.BackOfficeCollectionView"
    edit_node_class = "woost.controllers.backoffice.editstack.EditNode"
    edit_view = "woost.views.BackOfficeFieldsView"
    edit_form = "woost.views.ContentForm"
    edit_controller = \
        "woost.controllers.backoffice.itemfieldscontroller." \
        "ItemFieldsController"

    __deleted = False

    @getter
    def is_deleted(self):
        return self.__deleted

    # Indexing
    #--------------------------------------------------------------------------

    # Make sure draft copies' members don't get indexed
    def _should_index_member(self, member):
        return PersistentObject._should_index_member(self, member) and (
            member.primary or self.draft_source is None
        )

    def _should_index_member_full_text(self, member):
        return PersistentObject._should_index_member_full_text(self, member) \
            and self.draft_source is None

     # When validating unique members, ignore conflicts with the draft source
    def _counts_as_duplicate(self, other):
        return PersistentObject._counts_as_duplicate(self, other) \
            and other is not self.draft_source \
            and self is not other.draft_source

    # Last change timestamp
    #--------------------------------------------------------------------------
    @classmethod
    def get_last_instance_change(cls):
        max_value = datastore.root.get(cls.full_name + ".last_instance_change")
        return None if max_value is None else max_value.value

    @classmethod
    def set_last_instance_change(cls, last_change):
        for cls in cls.__mro__:
            if hasattr(cls, "set_last_instance_change"):
                key = cls.full_name + ".last_instance_change"
                max_value = datastore.root.get(key)
                if max_value is None:
                    datastore.root[key] = max_value = MaxValue(last_change)
                else:
                    max_value.value = last_change

    # Versioning
    #--------------------------------------------------------------------------
    versioned = True

    changes = schema.Collection(
        required = True,
        versioned = False,
        editable = False,
        items = "woost.models.Change",
        bidirectional = True,
        visible = False
    )

    creation_time = schema.DateTime(
        versioned = False,
        editable = False,
        member_group = "administration",
        indexed = True
    )

    last_update_time = schema.DateTime(
        indexed = True,
        versioned = False,
        editable = False,
        member_group = "administration"
    )

    is_draft = schema.Boolean(
        required = True,
        default = False,
        indexed = True,
        listed_by_default = False,
        editable = False,
        versioned = False,
        member_group = "administration"
    )

    draft_source = schema.Reference(
        type = "woost.models.Item",
        related_key = "drafts",
        bidirectional = True,
        editable = False,
        listed_by_default = False,
        indexed = True,
        versioned = False,
        member_group = "administration"
    )

    drafts = schema.Collection(
        items = "woost.models.Item",
        related_key = "draft_source",
        bidirectional = True,
        cascade_delete = True,
        editable = False,
        versioned = False,
        searchable = False,
        member_group = "administration"
    )

    _draft_count = 0
    _draft_id = None

    @getter
    def draft_id(self):
        """A numerical identifier for draft copies, guaranteed to be unique
        among their source item.
        @type: int
        """
        return self._draft_id

    def make_draft(self):
        """Creates a new draft copy of the item. Subclasses can tweak the copy
        process by overriding either this method or L{get_draft_adapter} (for
        example, to exclude one or more members).

        @return: The draft copy of the item.
        @rtype: L{Item}
        """
        draft = self.__class__(bidirectional = False)
        draft.bidirectional = True
        draft.draft_source = self
        draft.is_draft = True
        draft.bidirectional = False

        self._draft_count += 1
        draft._draft_id = self._draft_count

        adapter = self.get_draft_adapter()
        adapter.export_object(
            self,
            draft,
            source_schema = self.__class__,
            source_accessor = schema.SchemaObjectAccessor,
            target_accessor = schema.SchemaObjectAccessor
        )

        return draft

    draft_confirmation = Event(doc = """
        An event triggered just before a draft is confirmed.
        """)

    def confirm_draft(self):
        """Confirms a draft. On draft copies, this applies all the changes made
        by the draft to its source element, and deletes the draft. On brand new
        drafts, the item itself simply drops its draft status, and otherwise
        remains the same.

        @raise ValueError: Raised if the item is not a draft.
        """
        if not self.is_draft:
            raise ValueError("confirm_draft() must be called on a draft")

        self.draft_confirmation()

        if self.draft_source is None:
            self.bidirectional = True
            self.is_draft = False
        else:
            adapter = self.get_draft_adapter()
            adapter.source_accessor = schema.SchemaObjectAccessor
            adapter.target_accessor = schema.SchemaObjectAccessor
            adapter.import_object(
                self,
                self.draft_source,
                source_schema = self.__class__
            )
            self.delete()

    @classmethod
    def get_draft_adapter(cls):
        """Produces an adapter that defines the copy process used by the
        L{make_draft} method in order to produce draft copies of the item.

        @return: An adapter with all the rules required to obtain a draft copy
            of the item.
        @rtype: L{Adapter<cocktail.schema.adapter.Adapter>}
        """
        adapter = schema.Adapter()
        adapter.collection_copy_mode = schema.shallow
        adapter.exclude([
            member.name
            for member in cls.members().itervalues()
            if cls._should_exclude_in_draft(member)
        ] + ["owner"])
        return adapter

    @classmethod
    def _should_exclude_in_draft(cls, member):
        return not member.editable or not member.visible

    @classmethod
    def _create_translation_schema(cls, members):
        members["versioned"] = False
        PersistentObject._create_translation_schema.im_func(cls, members)

    @classmethod
    def _add_member(cls, member):
        if member.name == "translations":
            member.editable = False
            member.searchable = False
        PersistentClass._add_member(cls, member)

    def _get_revision_state(self):
        """Produces a dictionary with the values for the item's versioned
        members. The value of translated members is represented using a
        (language, translated value) mapping.

        @return: The item's current state.
        @rtype: dict
        """

        # Store the item state for the revision
        state = PersistentMapping()

        for key, member in self.__class__.members().iteritems():

            if not member.versioned:
                continue

            if member.translated:
                value = dict(
                    (language, translation.get(key))
                    for language, translation in self.translations.iteritems()
                )
            else:
                value = self.get(key)

                # Make a copy of mutable objects
                if isinstance(
                    value, (list, set, ListWrapper, SetWrapper)
                ):
                    value = list(value)

            state[key] = value

        return state

    # Item insertion overriden to make it versioning aware
    @event_handler
    def handle_inserting(cls, event):

        item = event.source
        now = datetime.now()
        item.creation_time = now
        item.last_update_time = now
        item.set_last_instance_change(now)
        item.__deleted = False

        if not item.is_draft and item.__class__.versioned:
            changeset = ChangeSet.current

            if changeset:
                change = Change()
                change.action = Action.get_instance(identifier = "create")
                change.target = item
                change.changed_members = set(
                    member.name
                    for member in item.__class__.members().itervalues()
                    if member.versioned
                )
                change.item_state = item._get_revision_state()
                change.changeset = changeset
                changeset.changes[item.id] = change

                if item.author is None:
                    item.author = changeset.author

                if item.owner is None:
                    item.owner = changeset.author

                change.insert(event.inserted_objects)

    # Extend item modification to make it versioning aware
    @event_handler
    def handle_changed(cls, event):

        item = event.source
        now = None

        if item.is_inserted:
            now = datetime.now()
            item.set_last_instance_change(now)

        if getattr(item, "_v_initializing", False) \
        or not event.member.versioned \
        or not item.is_inserted \
        or item.is_draft \
        or not item.__class__.versioned:
            return

        changeset = ChangeSet.current

        if changeset:

            member_name = event.member.name
            language = event.language
            change = changeset.changes.get(item.id)

            if change is None:
                action_type = "modify"
                change = Change()
                change.action = Action.get_instance(identifier = action_type)
                change.target = item
                change.changed_members = set()
                change.item_state = item._get_revision_state()
                change.changeset = changeset
                changeset.changes[item.id] = change
                if event.member.affects_last_update_time:
                    item.last_update_time = now or datetime.now()
                change.insert()
            else:
                action_type = change.action.identifier

            if action_type == "modify":
                change.changed_members.add(member_name)

            if action_type in ("create", "modify"):
                value = event.value

                # Make a copy of mutable objects
                if isinstance(
                    value, (list, set, ListWrapper, SetWrapper)
                ):
                    value = list(value)

                if language:
                    change.item_state[member_name][language] = value
                else:
                    change.item_state[member_name] = value
        elif event.member.affects_last_update_time:
            item.last_update_time = datetime.now()

    @event_handler
    def handle_deleting(cls, event):

        item = event.source

        # Update the last time of modification for the item
        now = datetime.now()
        item.set_last_instance_change(now)
        item.last_update_time = now

        if not item.is_draft and item.__class__.versioned:
            changeset = ChangeSet.current

            # Add a revision for the delete operation
            if changeset:
                change = changeset.changes.get(item.id)

                if change and change.action.identifier != "delete":
                    del changeset.changes[item.id]

                if change is None \
                or change.action.identifier not in ("create", "delete"):
                    change = Change()
                    change.action = Action.get_instance(identifier = "delete")
                    change.target = item
                    change.changeset = changeset
                    changeset.changes[item.id] = change
                    change.insert()

        item.__deleted = True

    @event_handler
    def handle_deleted(cls, event):

        item = event.source

        # Break the relation to the draft's source. This needs to be done
        # explicitly, because drafts are flagged as non bidirectional (to keep
        # their changes in isolation), which prevents automatic management of
        # referential integrity
        if item.draft_source is not None:
            item.draft_source.drafts.remove(item)

    _preserved_members = frozenset([changes])

    def _should_cascade_delete(self, member):
        return member.cascade_delete and not self.is_draft

    def _should_erase_member(self, member):
        return PersistentObject._should_erase_member(self, member) \
            and member not in self._preserved_members \
            and member is not self.__class__.draft_source

    # Ownership and authorship
    #--------------------------------------------------------------------------
    author = schema.Reference(
        indexed = True,
        editable = False,
        type = "woost.models.User",
        listed_by_default = False,
        member_group = "administration"
    )

    owner = schema.Reference(
        indexed = True,
        type = "woost.models.User",
        listed_by_default = False,
        member_group = "administration"
    )


Item.id.versioned = False
Item.id.editable = False
Item.id.listed_by_default = False
Item.id.member_group = "administration"
Item.changes.visible = False




#-*- coding: utf-8 -*-
"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
import buffet
import smtplib
from mimetypes import guess_type
from email.mime.text import MIMEText
from email.MIMEMultipart import MIMEMultipart
from email.MIMEImage import MIMEImage
from email.MIMEBase import MIMEBase
from email.Header import Header
from email.Utils import formatdate, parseaddr, formataddr
from email import Encoders
from cocktail import schema
from woost.models import Item, Site, File


class EmailTemplate(Item):

    encoding = "utf-8"

    members_order = [
        "title",
        "mime_type",
        "sender",
        "receivers",
        "bcc",
        "template_engine",
        "subject",
        "body",
        "initialization_code"
    ]

    title = schema.String(
        listed_by_default = False,
        required = True,
        unique = True,
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True
    )

    mime_type = schema.String(
        required = True,
        default = "html",
        listed_by_default = False,
        enumeration = [
            "plain",
            "html"
        ],
        text_search = False
    )

    sender = schema.CodeBlock(
        language = "python"
    )

    receivers = schema.CodeBlock(
        language = "python",
        required = True
    )

    bcc = schema.CodeBlock(
        language = "python",
        listed_by_default = False
    )

    template_engine = schema.String(
        enumeration = buffet.available_engines.keys(),
        text_search = False,
        listed_by_default = False
    )

    subject = schema.String(
        translated = True,
        edit_control = "cocktail.html.TextArea"
    )

    body = schema.String(
        translated = True,
        listed_by_default = False,
        edit_control = "cocktail.html.TextArea"
    )

    initialization_code = schema.CodeBlock(
        language = "python"
    )

    def send(self, context = None):

        smtp_host = Site.main.smtp_host or "localhost"
        smtp_port = smtplib.SMTP_PORT
        smtp_user = Site.main.smtp_user
        smtp_password = Site.main.smtp_password

        if context is None:
            context = {}

        if context.get("attachments") is None:
            context["attachments"] = {}

        def eval_member(key):
            expr = self.get(key)
            return eval(expr, context.copy()) if expr else None

        # MIME block
        mime_type = self.mime_type
        pos = mime_type.find("/")

        if pos != -1:
            mime_type = mime_type[pos + 1:]

        # Custom initialization code
        init_code = self.initialization_code
        if init_code:
            exec init_code in context

        # Subject and body (templates)
        if self.template_engine:
            template_engine = buffet.available_engines[self.template_engine]
            engine = template_engine(
                options = {"mako.output_encoding": self.encoding}
            )

            def render(field_name):
                markup = self.get(field_name)
                if markup:
                    template = engine.load_template(
                        "EmailTemplate." + field_name,
                        self.get(field_name)
                    )
                    return engine.render(context, template = template)
                else:
                    return u""

            subject = render("subject").strip()
            body = render("body")
        else:
            subject = self.subject.encode(self.encoding)
            body = self.body.encode(self.encoding)

        message = MIMEText(body, _subtype = mime_type, _charset = self.encoding)

        # Attachments
        attachments = context.get("attachments")
        if attachments:
            attachments = dict(
                (cid, attachment)
                for cid, attachment in attachments.iteritems()
                if attachment is not None
            )
            if attachments:
                message_text = message
                message = MIMEMultipart("related")
                message.attach(message_text)

                for cid, attachment in attachments.iteritems():

                    if isinstance(attachment, File):
                        file_path = attachment.file_path
                        file_name = attachment.file_name
                        mime_type = attachment.mime_type
                    else:
                        file_path = attachment
                        file_name = os.path.basename(file_path)
                        mime_type_guess = guess_type(file_path)
                        if mime_type_guess:
                            mime_type = mime_type_guess[0]
                        else:
                            mime_type = "application/octet-stream"

                    main_type, sub_type = mime_type.split('/', 1)
                    message_attachment = MIMEBase(main_type, sub_type)
                    message_attachment.set_payload(open(file_path).read())
                    Encoders.encode_base64(message_attachment)
                    message_attachment.add_header("Content-ID", "<%s>" % cid)
                    message_attachment.add_header(
                        'Content-Disposition',
                        'attachment; filename="%s"' % file_name
                    )
                    message.attach(message_attachment)

        def format_email_address(address, encoding):
            name, address = parseaddr(address)
            name = Header(name, encoding).encode()
            address = address.encode('ascii')
            return formataddr((name, address))

         # Receivers (python expression)
        receivers = eval_member("receivers")
        if receivers:
            receivers = set(r.strip().encode(self.encoding) for r in receivers)

        if not receivers:
            return set()

        message["To"] = ", ".join([
            format_email_address(receiver, self.encoding)
            for receiver in receivers
        ])

        # Sender (python expression)
        sender = eval_member("sender")
        if sender:
            message['From'] = format_email_address(sender, self.encoding)

        # BCC (python expression)
        bcc = eval_member("bcc")
        if bcc:
            receivers.update(r.strip().encode(self.encoding) for r in bcc)

        if subject:
            message["Subject"] = Header(subject, self.encoding).encode()

        message["Date"] = formatdate()

        # Send the message
        smtp = smtplib.SMTP(smtp_host, smtp_port)
        if smtp_user and smtp_password:
            smtp.login(
                smtp_user.encode(self.encoding),
                smtp_password.encode(self.encoding)
            )
        smtp.sendmail(sender, list(receivers), message.as_string())
        smtp.quit()

        return receivers




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""

# Create the default asynchronous file uploader
from cocktail.controllers.asyncupload import AsyncUploader
async_uploader = AsyncUploader()
async_uploader.session_prefix = "woost.async_upload."




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# TextFile
#------------------------------------------------------------------------------
translations.define("TextFile",
    ca = u"Fitxer de text",
    es = u"Fichero de texto",
    en = u"Text file"
)

translations.define("TextFile-plural",
    ca = u"Fitxers de text",
    es = u"Ficheros de texto",
    en = u"Text files"
)

translations.define("TextFile.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title"
)

translations.define("TextFile.content",
    ca = u"Contingut",
    es = u"Contenido",
    en = u"Content"
)

# TextFileController
#------------------------------------------------------------------------------
translations.define("woost.extensions.textfile.controller.title",
    ca = u"Controlador de fitxer de text",
    es = u"Controlador de fichero de texto",
    en = u"Text file controller"
)




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			Jun 2009
"""
from unittest import TestCase
from cocktail.tests.persistence.tempstoragemixin import TempStorageMixin

class BaseTestCase(TempStorageMixin, TestCase):

    def setUp(self):

        from woost.models import Site, Action, User, Role
        from woost.models.trigger import set_triggers_enabled

        set_triggers_enabled(False)

        TempStorageMixin.setUp(self)

        # Actions
        self.create_action = Action(identifier = "create")
        self.create_action.insert()

        self.read_action = Action(identifier = "read")
        self.read_action.insert()

        self.modify_action = Action(identifier = "modify")
        self.modify_action.insert()

        self.delete_action = Action(identifier = "delete")
        self.delete_action.insert()

        self.confirm_draft_action = Action(identifier = "confirm_draft")
        self.confirm_draft_action.insert()

        # Site
        self.site = Site(qname = "woost.main_site")
        self.site.insert()

        # Roles and users
        self.anonymous_role = Role(qname = "woost.anonymous")
        self.anonymous_role.insert()

        self.anonymous_user = User(qname = "woost.anonymous_user")
        self.anonymous_user.roles.append(self.anonymous_role)
        self.anonymous_user.insert()

        self.everybody_role = Role(qname = "woost.everybody")
        self.everybody_role.insert()

        self.authenticated_role = Role(qname = "woost.authenticated")
        self.authenticated_role.insert()

        set_triggers_enabled(True)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
from cocktail.modeling import extend, call_base, empty_list
from cocktail.html import templates
from woost.models import Site, Document

TreeContentView = templates.get_class("woost.views.TreeContentView")

class PublishableTreeContentView(TreeContentView):

    children_collection = Document.children

    def __init__(self, *args, **kwargs):
        TreeContentView.__init__(self, *args, **kwargs)
        home = Site.main.home
        self.root = [home] if home is not None else []




#-*- coding: utf-8 -*-
"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			November 2009
"""
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.translations import translations
from cocktail.tests.seleniumtester import selenium_test, browser
from woost.models import Item, Publishable, Template
from woost.tests.test_selenium import admin_login


class EditTestModel(Item):

    description = schema.String(
        required = True
    )

    part = schema.Reference(
        bidirectional = True,
        integral = True
    )

    container = schema.Reference(
        bidirectional = True
    )

EditTestModel.part.type = EditTestModel
EditTestModel.container.type = EditTestModel


class ItemSelectorTestCase(object):

    @selenium_test
    def test_select(self):

        template = list(Template.select())[0]

        browser.open("/en/cms/content/new/?item_type=woost.models.publishable.Publishable")
        admin_login()

        assert not browser.is_element_present(
            "css=.template_field .control .new"
        )

        assert not browser.is_element_present(
            "css=.template_field .control .unlink"
        )

        assert not browser.is_element_present(
            "css=.template_field .control .delete"
        )

        browser.click("css=.template_field .control .select")
        browser.wait_for_page_to_load(10000)

        assert browser.is_visible("css=.collection_display #%d" % template.id)
        browser.click("css=.collection_display #%d" % template.id)
        browser.click("css=.ContentView .select_action")
        browser.wait_for_page_to_load(10000)

        assert translations(template, "en") \
            in browser.jquery_html(".template_field .control")

        assert browser.get_value("edited_item_template") == str(template.id)

    @selenium_test
    def test_unlink(self):

        template = list(Template.select())[0]

        publishable = Publishable()
        publishable.set("enabled", True, "en")
        publishable.template = template
        publishable.insert()
        datastore.commit()

        browser.open("/en/cms/content/%d/fields" % publishable.id)
        admin_login()

        assert not browser.is_element_present(
            "css=.template_field .control .new"
        )

        assert not browser.is_element_present(
            "css=.template_field .control .delete"
        )

        browser.click("css=.template_field .control .unlink")
        browser.wait_for_page_to_load(10000)

        val = browser.get_value("edited_item_template")

        assert not browser.get_value("edited_item_template")


class IntegralSelectorTestCase(object):

    @selenium_test
    def test_new(self):

        browser.open(
            "/en/cms/content/new/"
            "?item_type=woost.tests.test_selenium.test_edit.EditTestModel"
        )
        admin_login()

        assert not browser.is_element_present("css=.part_field .control .select")
        assert not browser.is_element_present("css=.part_field .control .edit")
        assert not browser.is_element_present("css=.part_field .control .delete")
        assert not browser.is_element_present("css=.part_field .control .unlink")

        browser.click("css=.part_field .control .new")
        browser.wait_for_page_to_load(10000)

        browser.type("edited_item_description", "Foo")
        browser.click("css=.save_action")
        browser.wait_for_page_to_load(10000)

        datastore.sync()
        part_id = int(browser.get_value("edited_item_part"))
        assert EditTestModel.require_instance(part_id).description == "Foo"

    @selenium_test
    def test_edit(self):

        datastore.sync()
        container = EditTestModel()
        container.description = "test_edit container"

        part = EditTestModel()
        part.description = "test_edit part"
        container.part = part

        container.insert()
        datastore.commit()

        browser.open("/en/cms/content/%d/fields" % container.id)
        admin_login()

        assert not browser.is_element_present("css=.part_field .control .select")
        assert not browser.is_element_present("css=.part_field .control .new")
        assert not browser.is_element_present("css=.part_field .control .unlink")
        assert browser.get_value("edited_item_part") == str(part.id)

        browser.click("css=.part_field .control .edit")
        browser.wait_for_page_to_load(10000)

        browser.type("edited_item_description", "modified test_edit part")
        browser.click("css=.save_action")
        browser.wait_for_page_to_load(10000)

        browser.click("css=.cancel_action")
        browser.wait_for_page_to_load(10000)

        datastore.sync()
        assert browser.get_value("edited_item_part") == str(part.id)
        assert part.description == "modified test_edit part"

    @selenium_test
    def test_delete(self):

        datastore.sync()
        container = EditTestModel()
        container.description = "test_edit container"

        part = EditTestModel()
        part.description = "test_edit part"
        container.part = part

        container.insert()
        datastore.commit()

        browser.open("/en/cms/content/%d/fields" % container.id)
        admin_login()

        assert not browser.is_element_present("css=.part_field .control .select")
        assert not browser.is_element_present("css=.part_field .control .new")
        assert not browser.is_element_present("css=.part_field .control .unlink")

        location = browser.get_location()
        browser.click("css=.part_field .control .delete")
        browser.wait_for_page_to_load(10000)
        assert browser.get_location() == location
        assert not browser.get_value("edited_item_part")

        datastore.sync()
        assert EditTestModel.get_instance(part.id) is part

        browser.click("css=.save_action")
        browser.wait_for_page_to_load(10000)
        datastore.sync()
        assert EditTestModel.get_instance(part.id) is None




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández Camps
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
from cocktail.persistence import datastore
from cocktail.translations import translations
from cocktail import schema
from cocktail.html import templates
from woost.models import Extension, Template

translations.define("ReCaptchaExtension",
    ca = u"ReCaptcha",
    es = u"ReCaptcha",
    en = u"ReCaptcha"
)

translations.define("ReCaptchaExtension-plural",
    ca = u"ReCaptcha",
    es = u"ReCaptcha",
    en = u"ReCaptcha"
)

translations.define("ReCaptchaExtension.public_key",
    ca = u"Clau pública",
    es = u"Clave pública",
    en = u"Public key"
)

translations.define("ReCaptchaExtension.private_key",
    ca = u"Clau privada",
    es = u"Clave privada",
    en = u"Private key"
)

translations.define("ReCaptchaExtension.theme",
    ca = u"Tema",
    es = u"Tema",
    en = u"Theme"
)

translations.define("ReCaptchaExtension.ssl",
    ca = u"SSL",
    es = u"SSL",
    en = u"SSL"
)

translations.define("ReCaptchaExtension.custom_theme_widget",
    ca = u"Id de l'element DOM (només pel tema custom)",
    es = u"Id del elemento DOM (sólo para el tema custom)",
    en = u"DOM element Id (only for custom theme)"
)

translations.define("ReCaptchaExtension.custom_template",
    ca = u"Plantilla",
    es = u"Plantilla",
    en = u"Template"
)

translations.define("ReCaptchaExtension.custom_group",
    ca = u"Tema personalitzat",
    es = u"Tema personalizado",
    en = u"Custom Theme"
)


class ReCaptchaExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Afegeix suport pel servei anti-bots reCAPTCHA.""",
            "ca"
        )
        self.set("description",
            u"""Añade soporte para el servicio anti-bots reCAPTCA.""",
            "es"
        )
        self.set("description",
            u"""Adds support for anti-bot reCAPTCHA service.""",
            "en"
        )

    def _load(self):

        # Import the extension's models
        from woost.extensions.recaptcha import (
            strings,
            schemarecaptchas
        )

        # Append additional members to the extension
        ReCaptchaExtension.members_order = [
            "public_key",
            "private_key",
            "theme",
            "custom_theme_widget",
            "custom_template",
            "ssl"
        ]

        ReCaptchaExtension.add_member(
            schema.String(
                "public_key",
                required = True,
                text_search = False
            )
        )

        ReCaptchaExtension.add_member(
            schema.String(
                "private_key",
                required = True,
                text_search = False
            )
        )

        ReCaptchaExtension.add_member(
            schema.String(
                "theme",
                required = True,
                enumeration = (
                    "red", "white", "blackglass", "clean", "custom"
                ),
                default = "red",
                text_search = False
            )
        )

        ReCaptchaExtension.add_member(
            schema.String(
                "custom_theme_widget",
                exclusive = ReCaptchaExtension.theme.equal("custom"),
                member_group = "custom_group",
                text_search = False
            )
        )

        ReCaptchaExtension.add_member(
            schema.String(
                "custom_template",
                exclusive = ReCaptchaExtension.theme.equal("custom"),
                member_group = "custom_group",
                default = "woost.extensions.recaptcha.RecaptchaCustomView",
                text_search = False
            )
        )

        ReCaptchaExtension.add_member(
            schema.Boolean(
                "ssl",
                default = False
            )
        )




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from cocktail import schema
from cocktail.schema.expressions import Self, ExclusionExpression
from woost.models.item import Item


class State(Item):
    """An item state, used to define workflows."""

    members_order = [
        "title",
        "outgoing_transitions",
        "incoming_transitions"
    ]

    title = schema.String(
        unique = True,
        required = True,
        descriptive = True,
        translated = True
    )

    items = schema.Collection(
        items = "woost.models.Item",
        bidirectional = True,
        related_key = "workflow_state",
        visible = False
    )

    outgoing_transitions = schema.Collection(
        items = "woost.extensions.workflow.transition.Transition",
        related_key = "source_state",
        bidirectional = True,
        integral = True
    )

    incoming_transitions = schema.Collection(
        items = "woost.extensions.workflow.transition.Transition",
        related_key = "target_state",
        bidirectional = True,
        editable = False,
        cascade_delete = True
    )


Item.add_member(
    schema.Reference(
        "workflow_state",
        type = "woost.extensions.workflow.state.State",
        related_key = "items",
        bidirectional = True,
        editable = False,
        search_control = "cocktail.html.DropdownSelector"
    )
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import sys
import cherrypy
from cocktail.events import event_handler
from cocktail.translations import (
    translations,
    language_context,
    get_language
)
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.controllers import (
    request_property,
    get_parameter,
    Location
)
from woost.models import File, Language, get_current_user
from woost.controllers.notifications import notify_user
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from woost.extensions.facebookpublication import FacebookPublicationExtension
from woost.extensions.facebookpublication.facebookpublicationtarget \
    import FacebookPublicationTarget
from woost.extensions.facebookpublication.facebookpublicationpermission \
    import FacebookPublicationPermission


class FacebookAlbumsController(BaseBackOfficeController):

    notice_summary_threshold = 4
    view_class = "woost.extensions.facebookpublication.FacebookAlbumsView"

    @request_property
    def form_schema(self):
        return schema.Schema("FacebookAlbumsForm", members = [
            schema.String("album_title",
                required = True
            ),
            schema.String("album_description",
                edit_control = "cocktail.html.TextArea"
            ),
            schema.Collection("subset",
                items = schema.Reference(
                    type = File,
                    relation_constraints = [File.resource_type.equal("image")],
                    required = True,
                    enumeration = lambda ctx: self.selection
                ),
                min = 1,
                default = schema.DynamicDefault(lambda: self.selection)
            ),
            schema.Collection("photo_languages",
                min = 1,
                items = schema.String(
                    required = True,
                    enumeration = lambda ctx: Language.codes,
                    translate_value = lambda value, language = None, **kwargs:
                        "" if not value
                           else translations(value, language, **kwargs)
                ),
                default = schema.DynamicDefault(lambda: Language.codes)
            ),
            schema.Boolean("generate_story",
                required = True,
                default = True
            ),
            schema.Collection("publication_targets",
                items = schema.Reference(
                    type = FacebookPublicationTarget,
                    required = True,
                    enumeration = lambda ctx: self.allowed_publication_targets
                ),
                min = 1,
                default = schema.DynamicDefault(
                    lambda: self.allowed_publication_targets
                )
            )
        ])

    @request_property
    def form_data(self):
        return get_parameter(
            self.form_schema,
            errors = "ignore",
            undefined = "set_none" if self.submitted else "set_default",
            implicit_booleans = self.submitted
        )

    @request_property
    def form_errors(self):
        return schema.ErrorList(
            []
            if not self.submitted
            else self.form_schema.get_errors(self.form_data)
        )

    @event_handler
    def handle_before_request(cls, e):

        controller = e.source

        if not controller.allowed_publication_targets:
            raise cherrypy.HTTPError(403, "Forbidden")

    @request_property
    def allowed_publication_targets(self):

        from woost.extensions.facebookpublication \
            import FacebookPublicationExtension

        user = get_current_user()

        return [fb_target
                for fb_target in FacebookPublicationExtension.instance.targets
                if fb_target.auth_token
                and all(
                    user.has_permission(
                        FacebookPublicationPermission,
                        target = publishable,
                        publication_target = fb_target
                    )
                    for publishable in self.selection
                )]

    @request_property
    def selection(self):
        return get_parameter(
            schema.Collection("selection",
                items = schema.Reference(
                    type = File,
                    required = True
                ),
                min = 1
            ),
            errors = "raise"
        )

    @request_property
    def submitted(self):
        return cherrypy.request.method == "POST"

    @request_property
    def valid(self):
        return self.action != "publish" \
            or self.form_schema.validate(self.form_data)

    @request_property
    def action(self):
        return cherrypy.request.params.get("action")

    def submit(self):

        if self.action == "close":
            self.go_back()

        form_data = self.form_data
        subset = form_data["subset"]
        album_title = form_data["album_title"]
        album_description = form_data["album_description"]
        photo_languages = set(form_data["photo_languages"])
        generate_story = form_data["generate_story"]
        targets = form_data["publication_targets"]

        results = self.results

        for target in targets:
            album_data = None
            try:
                album_data = target.publish_album(
                    album_title,
                    subset,
                    album_description = album_description,
                    photo_languages = photo_languages,
                    generate_story = generate_story
                )
            except Exception, ex:
                results.append((target, album_data, ex))
            else:
                results.append((target, album_data, None))

    @request_property
    def results(self):
        return []

    @request_property
    def output(self):
        output = BaseBackOfficeController.output(self)
        output.update(
            form_schema = self.form_schema,
            form_data = self.form_data,
            form_errors = self.form_errors,
            results = self.results
        )
        return output




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Publishable
from woost.extensions.blocks.block import Block


class LinksBlock(Block):

    instantiable = True
    view_class = "woost.extensions.blocks.LinksBox"

    links = schema.Collection(
        items = schema.Reference(type = Publishable),
        related_end = schema.Collection(),
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.links = self.links




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
from datetime import datetime
from cocktail.modeling import getter
from cocktail.events import event_handler
from cocktail.pkgutils import import_object
from cocktail.translations import (
    translations,
    get_language,
    require_language
)
from cocktail import schema
from cocktail.schema.expressions import Expression
from cocktail.controllers import (
    make_uri,
    percent_encode_uri,
    Location
)
from woost.models.item import Item
from woost.models.language import Language
from woost.models.usersession import get_current_user
from woost.models.permission import (
    ReadPermission,
    ReadTranslationPermission,
    PermissionExpression
)
from woost.models.caching import CachingPolicy


class Publishable(Item):
    """Base class for all site elements suitable for publication."""

    instantiable = False

    # Backoffice customization
    preview_view = "woost.views.BackOfficePreviewView"
    preview_controller = "woost.controllers.backoffice." \
        "previewcontroller.PreviewController"
    edit_node_class = "woost.controllers.backoffice.publishableeditnode." \
        "PublishableEditNode"

    groups_order = ["navigation", "presentation", "publication"]

    members_order = [
        "inherit_resources",
        "mime_type",
        "resource_type",
        "encoding",
        "controller",
        "parent",
        "path",
        "full_path",
        "hidden",
        "login_page",
        "enabled",
        "translation_enabled",
        "start_date",
        "end_date",
        "requires_https",
        "caching_policies"
    ]

    mime_type = schema.String(
        required = True,
        default = "text/html",
        text_search = False,
        format = r"^[^/]+/[^/]+$",
        listed_by_default = False,
        member_group = "presentation"
    )

    resource_type = schema.String(
        indexed = True,
        text_search = False,
        editable = False,
        enumeration = (
            "document",
            "image",
            "audio",
            "video",
            "package",
            "html_resource",
            "other"
        ),
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(
                "woost.models.Publishable.resource_type " + value,
                 language,
                **kwargs
            ),
        listed_by_default = False,
        member_group = "presentation"
    )

    encoding = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "presentation",
        default = "utf-8"
    )

    controller = schema.Reference(
        type = "woost.models.Controller",
        indexed = True,
        bidirectional = True,
        listed_by_default = False,
        member_group = "presentation"
    )

    inherit_resources = schema.Boolean(
        listed_by_default = False,
        member_group = "presentation",
        default = True
    )

    def resolve_controller(self):
        if self.controller and self.controller.python_name:
            return import_object(self.controller.python_name)

    parent = schema.Reference(
        type = "woost.models.Document",
        bidirectional = True,
        related_key = "children",
        listed_by_default = False,
        member_group = "navigation"
    )

    path = schema.String(
        max = 1024,
        indexed = True,
        listed_by_default = False,
        text_search = False,
        member_group = "navigation"
    )

    full_path = schema.String(
        indexed = True,
        unique = True,
        editable = False,
        text_search = False,
        member_group = "navigation"
    )

    hidden = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "navigation"
    )

    login_page = schema.Reference(
        listed_by_default = False,
        member_group = "navigation"
    )

    per_language_publication = schema.Boolean(
        required = True,
        default = False,
        indexed = True,
        visible = False,
        member_group = "publication"
    )

    enabled = schema.Boolean(
        required = True,
        default = True,
        indexed = True,
        listed_by_default = False,
        member_group = "publication"
    )

    translation_enabled = schema.Boolean(
        required = True,
        default = True,
        translated = True,
        indexed = True,
        listed_by_default = False,
        member_group = "publication"
    )

    start_date = schema.DateTime(
        indexed = True,
        listed_by_default = False,
        member_group = "publication"
    )

    end_date = schema.DateTime(
        indexed = True,
        min = start_date,
        listed_by_default = False,
        member_group = "publication"
    )

    requires_https = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "publication"
    )

    caching_policies = schema.Collection(
        items = schema.Reference(type = CachingPolicy),
        bidirectional = True,
        integral = True,
        related_end = schema.Reference(),
        member_group = "publication"
    )

    def get_effective_caching_policy(self, **context):

        from woost.models import Site

        policies = [
            ((-policy.important, 1), policy)
            for policy in self.caching_policies
        ]
        policies.extend(
            ((-policy.important, 2), policy)
            for policy in Site.main.caching_policies
        )
        policies.sort()

        for criteria, policy in policies:
            if policy.applies_to(self, **context):
                return policy

    @event_handler
    def handle_changed(cls, event):

        member = event.member
        publishable = event.source

        if member.name == "path":
            publishable._update_path(publishable.parent, event.value)

        elif member.name == "parent":
            publishable._update_path(event.value, publishable.path)

        elif member.name == "mime_type":
            if event.value is None:
                publishable.resource_type = None
            else:
                publishable.resource_type = \
                    get_category_from_mime_type(event.value)

    def _update_path(self, parent, path):

        parent_path = parent and parent.full_path

        if parent_path and path:
            self.full_path = parent_path + "/" + path
        else:
            self.full_path = path

    def ascend_tree(self, include_self = False):
        """Iterate over the item's ancestors, moving towards the root of the
        document tree.

        @param include_self: Indicates if the object itself should be included
            in the iteration.
        @type include_self: bool

        @return: An iterable sequence of pages.
        @rtype: L{Document} iterable sequence
        """
        publishable = self if include_self else self.parent
        while publishable is not None:
            yield publishable
            publishable = publishable.parent

    def descends_from(self, page):
        """Indicates if the object descends from the given document.

        @param page: The hypothetical ancestor of the page.
        @type page: L{Document<woost.models.document.Document>}

        @return: True if the object is contained inside the given document or
            one of its descendants, or if it *is* the given document. False in
            any other case.
        @rtype: bool
        """
        ancestor = self

        while ancestor is not None:
            if ancestor is page:
                return True
            ancestor = ancestor.parent

        return False

    @getter
    def resources(self):
        """Iterates over all the resources that apply to the item.
        @type: L{Publishable}
        """
        return self.inherited_resources

    @getter
    def inherited_resources(self):
        """Iterates over all the inherited resources that apply to the item.
        @type: L{Publishable}
        """
        ancestry = []
        publishable = self

        while publishable.parent is not None and publishable.inherit_resources:
            ancestry.append(publishable.parent)
            publishable = publishable.parent

        ancestry.reverse()

        for publishable in ancestry:
            for resource in publishable.branch_resources:
                yield resource

    def is_current(self):
        now = datetime.now()
        return (self.start_date is None or self.start_date <= now) \
            and (self.end_date is None or self.end_date > now)

    def is_published(self, language = None):

        if self.is_draft:
            return False

        if self.per_language_publication:

            language = require_language(language)

            if not self.get("translation_enabled", language):
                return False

            site_language = Language.get_instance(iso_code = language)
            if site_language is None or not site_language.enabled:
                return False

        elif not self.enabled:
            return False

        if not self.is_current():
            return False

        return True

    def is_accessible(self, user = None, language = None):
        if user is None:
            user = get_current_user()

        return (
            self.is_published(language)
            and user.has_permission(
                ReadPermission,
                target = self
            )
            and (
                not self.per_language_publication
                or user.has_permission(
                    ReadTranslationPermission,
                    language = require_language(language)
                )
            )
        )

    @classmethod
    def select_accessible(cls, *args, **kwargs):
        return cls.select(filters = [
            IsAccessibleExpression(get_current_user())
        ]).select(*args, **kwargs)

    def get_uri(self,
        path = None,
        parameters = None,
        language = None,
        host = None,
        encode = True):

        from woost.models import Site
        uri = Site.main.get_path(self, language = language)

        if uri is not None:
            if self.per_language_publication:
                uri = make_uri(require_language(language), uri)

            if path:
                uri = make_uri(uri, *path)

            if parameters:
                uri = make_uri(uri, **parameters)

            uri = self.__fix_uri(uri, host, encode)

        return uri

    def get_image_uri(self,
        image_factory = None,
        parameters = None,
        encode = True,
        include_extension = True,
        host = None,):

        uri = make_uri("/images", self.id)

        if image_factory:
            uri = make_uri(uri, image_factory)

        if include_extension:
            from woost.models.rendering.formats import (
                formats_by_extension,
                extensions_by_format,
                default_format
            )
            ext = getattr(self, "file_extension", None)
            if ext is not None:
                ext = ext.lower()
            if ext is None \
            or ext.lstrip(".") not in formats_by_extension:
                ext = "." + extensions_by_format[default_format]
            uri += ext

        if parameters:
            uri = make_uri(uri, **parameters)

        return self.__fix_uri(uri, host, encode)

    def __fix_uri(self, uri, host, encode):

        if encode:
            uri = percent_encode_uri(uri)

        if host:
            if host == ".":
                location = Location.get_current_host()

                from woost.models import Site
                site = Site.main
                policy = site.https_policy

                if (
                    policy == "always"
                    or (
                        policy == "per_page" and (
                            self.requires_https
                            or not get_current_user().anonymous
                        )
                    )
                ):
                    location.scheme = "https"
                else:
                    location.scheme = "http"

                host = str(location)
            elif not "://" in host:
                host = "http://" + host

            uri = make_uri(host, uri)
        else:
            uri = make_uri("/", uri)

        return uri

Publishable.login_page.type = Publishable
Publishable.related_end = schema.Collection()


class IsPublishedExpression(Expression):
    """An expression that tests if items are published."""

    def eval(self, context, accessor = None):
        return context.is_published()

    def resolve_filter(self, query):

        def impl(dataset):


            # Exclude disabled items
            simple_pub = set(
                Publishable.per_language_publication.index.values(key = False)
            ).intersection(Publishable.enabled.index.values(key = True))

            language = get_language()
            site_language = Language.get_instance(iso_code = language)
            per_language_pub = set(
                Publishable.per_language_publication.index.values(key = True)
            )

            if site_language is None or not site_language.enabled:
                dataset.intersection_update(simple_pub)
                dataset.difference_update(per_language_pub)
            else:
                per_language_pub.intersection_update(
                    Publishable.translation_enabled.index.values(
                        key = (language, True)
                    )
                )
                dataset.intersection_update(simple_pub | per_language_pub)

            # Exclude drafts
            dataset.difference_update(Item.is_draft.index.values(key = True))

            # Remove items outside their publication window
            now = datetime.now()
            dataset.difference_update(
                Publishable.start_date.index.values(
                    min = Publishable.start_date.get_index_value(now),
                    exclude_min = True
                )
            )
            dataset.difference_update(
                Publishable.end_date.index.values(
                    min = None,
                    exclude_min = True,
                    max = Publishable.end_date.get_index_value(now),
                    exclude_max = True
                )
            )

            return dataset

        return ((-1, 1), impl)


class IsAccessibleExpression(Expression):
    """An expression that tests that items can be accessed by a user.

    The expression checks both the publication state of the item and the
    read permissions for the specified user.

    @ivar user: The user that accesses the items.
    @type user: L{User<woost.models.user.User>}
    """
    def __init__(self, user = None):
        Expression.__init__(self)
        self.user = user

    def eval(self, context, accessor = None):
        return context.is_accessible(user = self.user)

    def resolve_filter(self, query):

        def impl(dataset):
            user = self.user or get_current_user()
            access_expr = PermissionExpression(user, ReadPermission)
            published_expr = IsPublishedExpression()
            dataset = access_expr.resolve_filter(query)[1](dataset)
            dataset = published_expr.resolve_filter(query)[1](dataset)

            if not user.has_permission(
                ReadTranslationPermission,
                language = require_language()
            ):
                dataset.difference_update(
                    Publishable.per_language_publication.index.values(
                        key = True
                    )
                )

            return dataset

        return ((-1, 1), impl)


mime_type_categories = {}

for category, mime_types in (
    ("text/plain", ("text",)),
    ("html_resource", (
        "text/css",
        "text/javascript",
        "text/ecmascript",
        "application/javascript",
        "application/ecmascript"
    )),
    ("document", (
        "application/vnd.oasis.opendocument.text",
        "application/vnd.oasis.opendocument.spreadsheet",
        "application/vnd.oasis.opendocument.presentation",
        "application/msword",
        "application/msexcel",
        "application/msaccess",
        "application/mspowerpoint",
        "application/mswrite",
        "application/vnd.ms-excel",
        "application/vnd.ms-access",
        "application/vnd.ms-powerpoint",
        "application/vnd.ms-project",
        "application/vnd.ms-works",
        "application/vnd.ms-xpsdocument",
        "application/rtf",
        "application/pdf",
        "application/x-pdf",
        "application/postscript",
        "application/x-latex",
        "application/vnd.oasis.opendocument.database"
    )),
    ("package", (
        "application/zip",
        "application/x-rar-compressed",
        "application/x-tar",
        "application/x-gtar",
        "application/x-gzip",
        "application/x-bzip",
        "application/x-stuffit",
        "vnd.ms-cab-compressed"
    ))
):
    for mime_type in mime_types:
        mime_type_categories[mime_type] = category

def get_category_from_mime_type(mime_type):
    """Obtains the file category that best matches the indicated MIME type.

    @param mime_type: The MIME type to get the category for.
    @type mime_type: str

    @return: A string identifier with the category matching the indicated
        MIME type (one of 'document', 'image', 'audio', 'video', 'package',
        'html_resource' or 'other').
    @rtype: str
    """
    pos = mime_type.find("/")

    if pos != -1:
        prefix = mime_type[:pos]

        if prefix in ("image", "audio", "video"):
            return prefix

    return mime_type_categories.get(mime_type, "other")




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import re
from cStringIO import StringIO
import cherrypy
from cocktail import schema
from cocktail.controllers import get_parameter
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from woost.extensions.workflow.state import State
from woost.extensions.workflow.graph import workflow_graph, render_graph

title_expr = re.compile("<title>.*</title>")


class WorkflowGraphController(BaseBackOfficeController):

    def __call__(self, **kwargs):

        cherrypy.response.headers["Content-Type"] = "image/svg+xml"

        states = get_parameter(
            schema.Collection("states",
                items = schema.Reference(type = State),
                default = schema.DynamicDefault(
                    lambda: list(State.select())
                )
            )
        )

        canvas = StringIO()
        graph = workflow_graph(states)
        render_graph(graph, canvas, "svg")
        svg = canvas.getvalue()

        # Add a stylesheet
        pos = svg.find("<svg")
        if pos != -1:
            svg = (
                svg[:pos]
                + "<?xml-stylesheet href='/resources/styles/workflow-graph.css' type='text/css'?>"
                + svg[pos:]
            )

        # Add a script
        pos = svg.rfind("</svg>")
        if pos != -1:
            svg = (
                svg[:pos]
                + "<script type='application/ecmascript' xlink:href='/resources/scripts/workflow-graph.js'/>"
                + svg[pos:]
            )

        # Remove pointless title attributes
        svg = title_expr.sub("", svg)

        return svg




#-*- coding: utf-8 -*-
"""

@author:		Marc Pérez
@contact:		marc.perez@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""

from cocktail.translations import translations

from cocktail.html.pager import Pager
from cocktail.html import Element

class GooglePager(Pager):

    def create_button(self, name):
        button = Element("a")
        button.add_class(name)
        if name == "next":
            button.append(translations("googlesearch next"))
        if name == "previous":
            button.append(translations("googlesearch previous"))
        return button





#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from unittest import TestCase
from cocktail.tests.persistence.tempstoragemixin import TempStorageMixin


class InitializationTestCase(TempStorageMixin, TestCase):

    def test_init_site(self):
        from woost.models.initialization import init_site
        init_site()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.extensions.blocks.containerblock import ContainerBlock


class SlideShowBlock(ContainerBlock):

    instantiable = True
    view_class = "cocktail.html.SlideShow"

    groups_order = ["content", "transition_settings"]

    members_order = [
        "autoplay",
        "interval",
        "transition_duration",
        "navigation_controls"
    ]

    autoplay = schema.Boolean(
        required = True,
        default = True,
        member_group = "transition_settings"
    )

    interval = schema.Integer(
        required = autoplay,
        default = 3000,
        min = 0,
        member_group = "transition_settings"
    )

    transition_duration = schema.Integer(
        required = True,
        default = 500,
        min = 0,
        member_group = "transition_settings"
    )

    navigation_controls = schema.Boolean(
        required = True,
        default = False,
        member_group = "content"
    )

    def init_view(self, view):
        ContainerBlock.init_view(self, view)
        view.block_content = view.slides
        view.autoplay = self.autoplay
        view.interval = self.interval
        view.transition_duration = self.transition_duration
        view.navigation_controls = self.navigation_controls




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from tpv.sis import SISPaymentGateway as Implementation
from cocktail import schema
from cocktail.translations import translations
from cocktail.modeling import getter
from cocktail.controllers import context
from cocktail.controllers.location import Location
from woost.models import Document
from woost.extensions.payments.paymentgateway import PaymentGateway


class SISPaymentGateway(PaymentGateway, Implementation):

    members_order = [
        "merchant_name", "merchant_code", "merchant_terminal",
        "merchant_secret_key", "payment_successful_page", "payment_failed_page"
    ]

    instantiable = True

    default_label = schema.DynamicDefault(
        lambda: translations("SISPaymentGateway.label default")
    )

    merchant_code = schema.String(
        required = True,
        shadows_attribute = True,
        text_search = False
    )

    merchant_name = schema.String(
        shadows_attribute = True,
        text_search = False
    )

    merchant_terminal = schema.String(
        required = True,
        shadows_attribute = True,
        text_search = False
    )

    merchant_secret_key = schema.String(
        required = True,
        shadows_attribute = True,
        text_search = False
    )

    payment_successful_page = schema.Reference(
        type = Document,
        related_end = schema.Reference()
    )

    payment_failed_page = schema.Reference(
        type = Document,
        related_end = schema.Reference()
    )

    def _base_url(self):
        location = Location.get_current()
        location.relative = False
        location.path_info = context["cms"].application_uri()
        location.query_string = None

        return unicode(location)[:-1]

    @getter
    def notification_url(self):
        return self._base_url() + context["cms"].application_uri("payment_notification")

    @getter
    def payment_successful_url(self):

        if self.payment_successful_page:
            return self._base_url() \
                + context["cms"].uri(self.payment_successful_page) \
                + "?payment_id=%(payment_id)s"

        return None

    @getter
    def payment_failed_url(self):

        if self.payment_failed_page:
            return self._base_url() \
                + context["cms"].uri(self.payment_failed_page) \
                + "?payment_id=%(payment_id)s"

        return None




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.persistence import MigrationStep

step = MigrationStep(
    "rename VimeoVideo.tags to VimeoVideo.vimeo_tags"
)

step.rename_member(
    "woost.extensions.vimeo.video.VimeoVideo",
    "tags",
    "vimeo_tags"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.events import when
from cocktail.persistence import MigrationStep

step = MigrationStep(
    "rename Block.Container_blocks to Block.containers"
)

step.rename_member(
    "woost.extensions.blocks.block.Block",
    "Container_blocks",
    "containers"
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
import cherrypy
from cocktail.modeling import cached_getter
from cocktail import schema
from cocktail.controllers import get_parameter
from cocktail.controllers.usercollection import UserCollection
from woost.extensions.shop.shopcontroller import ShopController
from woost.extensions.shop.product import Product


class ProductsController(ShopController):
    """A controller used to browse a shop's product catalog."""

    def resolve(self, path):

        # Product detail
        if path:
            product_id = path.pop(0)

            try:
                product_id = int(product_id)
                product = Product.require_instance(product_id)
            except:
                raise cherrypy.NotFound()

            return self.get_detail_controller(product)

        # Product listing
        return self

    def get_detail_controller(self, product):
        return ProductDetailController(product)

    @cached_getter
    def user_collection(self):
        user_collection = UserCollection(Product)
        return user_collection

    @cached_getter
    def output(self):
        output = ShopController.output(self)
        output["user_collection"] = self.user_collection
        return output


class ProductDetailController(ShopController):
    """A controller that provides a detailed view over a single product.

    @ivar product: The selected product.
    @type product: L{Product<woost.extensions.shop.product.Product>}
    """

    def __init__(self, product, *args, **kwargs):
        ShopController.__init__(self, *args, **kwargs)
        self.product = product

    @cached_getter
    def output(self):
        output = ShopController.output(self)
        output["product"] = self.product
        return output




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.extensions.blocks.block import Block


class IFrameBlock(Block):

    instantiable = True
    view_class = "cocktail.html.Element"
    tag = "iframe"

    members_order = [
        "src",
        "width",
        "height"
    ]

    src = schema.URL(
        required = True,
        member_group = "content"
    )

    width = schema.Integer(
        min = 0,
        member_group = "content"
    )

    height = schema.Integer(
        min = 0,
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view["src"] = self.src
        view["frameborder"] = "0"

        if self.width:
            view["width"] = self.width

        if self.height:
            view["height"] = self.height




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""

class Module(object):

    def __init__(self, application):
        self.application = application




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
from contextlib import contextmanager
from cocktail.modeling import InstrumentedSet
from cocktail.events import when
from cocktail.pkgutils import import_object
from cocktail.translations import translations
from cocktail import schema
from cocktail.controllers.usercollection import UserCollection
from cocktail.schema.expressions import Expression
from woost.models.item import Item
from woost.models.language import Language
from woost.models.messagestyles import permission_doesnt_match_style
from woost.models.usersession import get_current_user
from woost.models.messagestyles import unauthorized_style


class Permission(Item):

    instantiable = False
    visible_from_root = False

    authorized = schema.Boolean(
        required = True,
        default = True
    )

    role = schema.Reference(
        type = "woost.models.Role",
        bidirectional = True,
        min = 1
    )

    def match(self, verbose = False):
        """Indicates if the permission matches the given context.

        @return: True if the permission matches, False otherwise.
        @rtype: bool
        """
        return True

    def __translate__(self, language, **kwargs):
        return translations(
            self.__class__.full_name + "-instance",
            language,
            instance = self,
            **kwargs
        ) or Item.__translate__(self, language, **kwargs)

    @classmethod
    def permission_not_found(cls, user, verbose = False, **context):
        if verbose:
            print unauthorized_style("unauthorized")
        return False


class ContentPermission(Permission):
    """Base class for permissions restricted to a subset of a content type."""

    edit_controller = \
        "woost.controllers.backoffice.contentpermissionfieldscontroller." \
        "ContentPermissionFieldsController"
    edit_view = "woost.views.ContentPermissionFields"

    matching_items = schema.Mapping(
        translate_value = lambda value, language = None, **kwargs:
            ""
            if not value
            else translations(
                ContentPermission._get_user_collection(value).subset
            )
    )

    def match(self, target, verbose = False):

        query = self.select_items()

        if isinstance(target, type):
            if not issubclass(target, query.type):
                if verbose:
                    print permission_doesnt_match_style("type doesn't match"),
                return False
            elif not self.authorized and "filter" in self.matching_items:
                if verbose:
                    print permission_doesnt_match_style("partial restriction")
                return False
        else:
            if not issubclass(target.__class__, query.type):
                if verbose:
                    print permission_doesnt_match_style("type doesn't match"),
                return False

            for filter in query.filters:
                if not filter.eval(target):
                    if verbose:
                        print permission_doesnt_match_style(
                            "filter %s doesn't match" % filter
                        ),
                    return False

        return True

    def select_items(self, *args, **kwargs):

        subset = self._get_user_collection(self.matching_items).subset

        if args or kwargs:
            subset = subset.select(*args, **kwargs)

        return subset

    @classmethod
    def _get_user_collection(self, matching_items):
        user_collection = UserCollection(Item)
        user_collection.allow_paging = False
        user_collection.allow_member_selection = False
        user_collection.allow_language_selection = False
        user_collection.params.source = matching_items.get
        user_collection.available_languages = Language.codes
        return user_collection


class ReadPermission(ContentPermission):
    """Permission to list and view instances of a content type."""
    instantiable = True


class CreatePermission(ContentPermission):
    """Permission to create new instances of a content type."""
    instantiable = True


class ModifyPermission(ContentPermission):
    """Permission to modify existing instances of a content type."""
    instantiable = True


class DeletePermission(ContentPermission):
    """Permission to delete instances of a content type."""
    instantiable = True


class ConfirmDraftPermission(ContentPermission):
    """Permission to confirm drafts of instances of a content type."""
    instantiable = True


class RenderPermission(ContentPermission):
    """Permission to obtain images representing instances of a content type."""
    instantiable = True

    def _image_factories_enumeration(ctx):
        from woost.models.rendering.factories import image_factories
        return image_factories.keys()

    image_factories = schema.Collection(
        items = schema.String(enumeration = _image_factories_enumeration),
        searchable = False
    )

    del _image_factories_enumeration

    def match(self, target, image_factory, verbose = False):

        if self.image_factories and image_factory not in self.image_factories:
            print permission_doesnt_match_style("image_factory doesn't match")
            return False

        return ContentPermission.match(self, target, verbose)

    @classmethod
    def permission_not_found(cls, user, verbose = False, **context):
        # If no specific render permission is found, a read permission will do
        return user.has_permission(
            ReadPermission,
            target = context["target"],
            verbose = verbose
        )


class TranslationPermission(Permission):
    """Base class for permissions that restrict operations on languages."""

    matching_languages = schema.Collection(
        items = schema.String(
            enumeration = lambda ctx: Language.codes,
            translate_value = lambda value, language = None, **kwargs:
                u"" if not value else translations(value, language, **kwargs)
        )
    )

    def match(self, language, verbose = False):

        languages = self.matching_languages

        if languages and language not in languages:
            if verbose:
                print permission_doesnt_match_style("language doesn't match"),
            return False

        return True


class CreateTranslationPermission(TranslationPermission):
    """Permission to add new translations."""
    instantiable = True


class ReadTranslationPermission(TranslationPermission):
    """Permission to view values translated into certain languages."""
    instantiable = True


class ModifyTranslationPermission(TranslationPermission):
    """Permission to modify the values of an existing translation."""
    instantiable = True


class DeleteTranslationPermission(TranslationPermission):
    """Permission to delete translations."""
    instantiable = True

def _resolve_matching_member_reference(compound_name):
    pos = compound_name.rfind(".")
    type_full_name = compound_name[:pos]
    member_name = compound_name[pos+1:]
    cls = import_object(type_full_name)
    return cls[member_name]

def _eligible_members():
    for cls in [Item] + list(Item.derived_schemas()):
        for name, member in cls.members(recursive = False).iteritems():
            if member.visible and member.name != "translations":
                yield cls.full_name + "." + name


class MemberPermission(Permission):
    """Base class for permissions that restrict operations on members."""

    matching_members = schema.Collection(
        default_type = set,
        items = schema.String(
            enumeration = lambda ctx: set(_eligible_members()),
            translate_value = lambda value, language = None, **kwargs:
                ""
                if not value
                else translations(
                    _resolve_matching_member_reference(value),
                    language,
                    qualified = True
                )
        ),
        edit_control = "woost.views.MemberList"
    )

    def match(self, member, verbose = False):

        member = member.original_member.schema.full_name + "." + member.name
        members = self.matching_members

        if members and member not in members:
            if verbose:
                print permission_doesnt_match_style("member doesn't match"),
            return False

        return True

    def iter_members(self):
        for compound_name in self.matching_members:
            yield _resolve_matching_member_reference(compound_name)


class ReadMemberPermission(MemberPermission):
    """Permission to view the value of certain members."""
    instantiable = True


class ModifyMemberPermission(MemberPermission):
    """Permission to modify the value of certain members."""
    instantiable = True


class ReadHistoryPermission(Permission):
    """Permission to view item revisions."""
    instantiable = True


@contextmanager
def restricted_modification_context(
    item,
    user = None,
    member_subset = None,
    verbose = False
):
    """A context manager that restricts modifications to an item.

    @param item: The item to monitor.
    @type item: L{Item<woost.models.item.Item>}

    @param user: The user that performs the modifications. If no user is
        provided, the user returned by
        L{get_current_user<woost.models.usersession.get_current_user>}
        will be used.
    @type user: L{User<woost.models.user.User>}

    @param verbose: Set to True to enable debug messages for the permission
        checks executed by this function.
    @type verbose: True

    @raise L{AuthorizationError<woost.models.user.AuthorizationError}:
        Raised if attempting to execute an action on the monitored item without
        the proper permission.
    """
    if user is None:
        user = get_current_user()

    if item.__class__.translated:
        starting_languages = set(item.translations.keys())
        modified_languages = set()

    # Modifying an existing item
    if item.is_inserted:
        is_new = False
        permission_type = ModifyPermission

        # Restrict access *before* the object is modified. This is only done on
        # existing objects, to make sure the current user is allowed to modify
        # them, taking into account constraints that may derive from the
        # object's present state. New objects, by definition, have no present
        # state, so the test is skipped.
        user.require_permission(
            ModifyPermission,
            target = item,
            verbose = verbose
        )

    # Creating a new item
    else:
        is_new = True
        permission_type = CreatePermission

    # Add an event listener to the edited item, to restrict changes to its
    # members
    @when(item.changed)
    def restrict_members(event):

        member = event.member

        # Require permission to modify the changed member
        if member_subset is None or member.name in member_subset:
            user.require_permission(
                ModifyMemberPermission,
                member = member,
                verbose = verbose
            )

        if member.translated:
            language = event.language

            # Require permission to create a new translation
            if is_new:
                if language not in starting_languages \
                and language not in modified_languages:
                    user.require_permission(
                        CreateTranslationPermission,
                        language = language,
                        verbose = verbose
                    )
                    modified_languages.add(language)

            # Require permission to modify an existing translation
            else:
                if language not in modified_languages:
                    user.require_permission(
                        ModifyTranslationPermission,
                        language = language,
                        verbose = verbose
                    )
                    modified_languages.add(language)

    # Try to modify the item
    try:
        yield None

    # Remove the added event listener
    finally:
        item.changed.remove(restrict_members)

    # Require permission to delete removed translations
    if item.__class__.translated:
        for language in starting_languages - set(item.translations):
            user.require_permission(
                DeleteTranslationPermission,
                language = language,
                verbose = verbose
            )

    # Restrict access *after* the object is modified, both for new and old
    # objects, to make sure the user is leaving the object in a state that
    # complies with all existing restrictions.
    user.require_permission(
        permission_type,
        target = item,
        verbose = verbose
    )

def delete_validating(item, user = None, deleted_set = None):

    if user is None:
        user = get_current_user()

    if deleted_set is None:
        class ValidatingDeletedSet(InstrumentedSet):
            def item_added(self, item):
                user.require_permission(DeletePermission, target = item)

        deleted_set = ValidatingDeletedSet()

    item.delete(deleted_set)
    return deleted_set


class PermissionExpression(Expression):
    """An schema expression that indicates if the specified user has permission
    over an element.
    """
    user = None
    permission_type = None

    def __init__(self, user, permission_type):
        self.user = user
        self.permission_type = permission_type

    def eval(self, context, accessor = None):
        return self.user.has_permission(self.permission_type, target = context)

    def resolve_filter(self, query):

        def impl(dataset):

            authorized_subset = set()
            queried_type = query.type

            for permission in reversed(list(
                self.user.iter_permissions(self.permission_type)
            )):
                permission_query = permission.select_items()

                if query.verbose:
                    permission_query.verbose = True
                    permission_query.nesting = query.nesting + 1

                if issubclass(queried_type, permission_query.type) \
                or issubclass(permission_query.type, queried_type):

                    permission_subset = permission_query.execute()

                    if permission.authorized:
                        authorized_subset.update(permission_subset)
                    else:
                        authorized_subset.difference_update(permission_subset)

            dataset.intersection_update(authorized_subset)
            return dataset

        return ((0, 0), impl)


class ChangeSetPermissionExpression(Expression):

    user = None

    def __init__(self, user):
        self.user = user

    def eval(self, context, accessor = None):
        return any(
            self.user.has_permission(
                ReadPermission,
                target = change.target
            )
            for change in context.changes.itervalues()
        )

    def resolve_filter(self, query):

        def impl(dataset):

            authorized_subset = set()

            for item in Item.select([
                PermissionExpression(self.user, ReadPermission)
            ]):
                for change in item.changes:
                    authorized_subset.add(change.changeset.id)

            dataset.intersection_update(authorized_subset)
            return dataset

        return ((0, 0), impl)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
from cocktail.translations import translations
from cocktail import schema
from cocktail.html import templates
from woost.models.messagestyles import permission_doesnt_match_style
from woost.models.permission import ContentPermission


class TransitionPermission(ContentPermission):
    """Permission to apply a state transition to an item."""

    instantiable = True

    def _transition_edit_control(parent, obj, member):
        display = templates.new("cocktail.html.DropdownSelector")
        display.empty_label = translations(
            "woost.extensions.workflow any transition"
        )
        return display

    transition = schema.Reference(
        type = "woost.extensions.workflow.transition.Transition",
        related_key = "transition_permissions",
        bidirectional = True,
        edit_control = _transition_edit_control
    )

    del _transition_edit_control

    def match(self, target, transition, verbose = False):

        if self.transition and transition is not self.transition:
            print permission_doesnt_match_style("transition doesn't match")
            return False

        return ContentPermission.match(self, target, verbose)

